/*************************************************
**Authors: Kevin Cartrette, Alex Lemke			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary: Bomb class which handles dealing with**
**killing enemies with a radial effect.			**
*************************************************/

#include "Bomb.h"
#include "Object.h"
#include "Vector2.h"
#include "ShapeDraw.h"
#include "EnemyManager.h"
#include "SoundManager.h"
#include "gamePlayVariables.h"

BombC::BombC()
{
}

BombC::~BombC()
{
}

void BombC::init(Vector3C initPosition, Vector3C initVelocity, double64_t initTimeToLive, double64_t initTimeBetweenUpdates)
{
	position.setValues(initPosition.x, initPosition.y);
	velocity.setValues(initVelocity.x, initVelocity.y);
	radius = TWO;
	timeToLive = initTimeToLive;
	currentTime = 0;
	totalTime = 0;
	timeBetweenUpdates = initTimeBetweenUpdates;
	active = true;
	emit = true;
}

bool BombC::update(double64_t milliseconds)
{
	currentTime += milliseconds;
	if(currentTime >= timeBetweenUpdates)
	{
		totalTime += currentTime;
		if(totalTime >= timeToLive)
		{
			active = false;
			return active;
		}

		radius += (float_t)(0.5f * currentTime);

		EnemyManagerC::GetInstance()->checkEnemyRadialCollision(radius,position,true);

		currentTime = 0;
	}
	return active;
}

void BombC::render()
{
	if(emit == true)
	{
		DrawCircle(radius,position.x, position.y,0.8f,0.8f,1.0f,false);
		DrawCircle(radius+10,position.x, position.y,0.8f,0.8f,1.0f,false);
		DrawCircle(radius+20,position.x, position.y,0.8f,0.8f,1.0f,false);
		DrawCircle(radius+30,position.x, position.y,0.8f,0.8f,1.0f,false);
	}
}

void BombC::shutdown()
{
	delete this;
}