/*************************************************
**Authors: Kevin Cartrette, Alex Lemke			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary: Bullet class which is used to detect **
**if the player is hitting an enemy with		**
**projectiles.									**
*************************************************/

#include <windows.h>											
#include <stdio.h>												
#include <stdarg.h>											
#include <math.h>												
#include "gl\gl.h"											
#include "gl\glu.h"												
#include "gl\glut.h"
#include "baseTypes.h"
#include "ShapeDraw.h"
#include "collInfo.h"
#include "object.h"
#include "field.h"
#include "fieldmanager.h"
#include "random.h"
#include "stateManager.h"
#include "inputmanager.h"
#include "EnemyManager.h"
#include "SpriteManager.h"
#include "SoundManager.h"
#include "ParticleManager.h"
#include "Bullet.h"


BulletC::BulletC()
{
    mEnabled = true;
    mCollInfo.shape = CollInfoC::SHAPE_CIRCLE;

	mRadius = 3.0f;
	mySprite.setUp(3, 0,0,0, 20, 20,0,4,4,8,8);
	mySprite.setCentered(true);
	mEnabled = false;
};

BulletC::~BulletC()
{
};

void BulletC::update(DWORD milliseconds)
{
	if(mEnabled){
		switch(StateManagerC::GetInstance()->getState())
		{
			case StateManagerC::GAMEPLAY:
				
				//Increase Position
				mPosition.x += mVelocity.x*milliseconds;
				mPosition.y += mVelocity.y*milliseconds;

				//Set Position
				mySprite.setPosition(mPosition.x, mPosition.y, 0.0f);
				//mySprite.setPosition(mPosition.x - mySprite.returnWidth()/2.0f, mPosition.y - mySprite.returnHeight()/2.0f, 0.0f);

				//Setting Rotation
				Vector3C straightUp;
				straightUp.setValues(0.0f, 1.0f);
				Vector3C velocityNormalzied;
				velocityNormalzied.setValues(mVelocity.x, mVelocity.y);
				
				velocityNormalzied.normalize();
				
				float_t angle = velocityNormalzied.angleBetween(straightUp);

				float negate = 1.0f;
				//
				if(mVelocity.x > 0.0f)
					mySprite.setRotation(angle * -180.0f / 3.1415926 + 90.0f);
				else	
					mySprite.setRotation(angle * 180.0f / 3.1415926 + 90.0f);

				DoCollisions();
				break;
		}
	}
}

void BulletC::DoCollisions()
{
	CollideField();
}

void BulletC::CollideField(void)
{
	bool hitHappened = false;
	FieldC *field = FieldManagerC::GetInstance()->getFieldPtr();

	float_t rightSide = field->getPosition()->x+((float_t)field->getWidth()/2.0f);
	float_t leftSide = field->getPosition()->x-((float_t)field->getWidth()/2.0f);
	float_t topSide = field->getPosition()->y+((float_t)field->getHeight()/2.0f);
	float_t bottomSide = field->getPosition()->y-((float_t)field->getHeight()/2.0f);
	
	//Box collision
	if(mPosition.x- mRadius <= leftSide  )
	{
		//mVelocity.x = -mVelocity.x;
		//mPosition.x = leftSide + mRadius;	
		mEnabled = false;
		hitHappened = true;
		
	}
	if(mPosition.x+ mRadius >= rightSide  )
	{
		//mVelocity.x = -mVelocity.x;
		//mPosition.x = rightSide - mRadius;
		mEnabled = false;
		hitHappened = true;
	}
	if(mPosition.y- mRadius <= bottomSide  )
	{
		//mVelocity.y = -mVelocity.y;
		//mPosition.y = bottomSide + mRadius;	
		mEnabled = false;
		hitHappened = true;
	}
	if(mPosition.y+ mRadius >= topSide  )
	{
		//mVelocity.y = -mVelocity.y;
		//mPosition.y = topSide - mRadius;
		mEnabled = false;
		hitHappened = true;
	}

	if(hitHappened == true)
	{
		Vector3C position(mPosition.x,mPosition.y,0);
		Vector3C velocity(5,5,0);
		Vector3C angularVelocity(2,2,0);
		SoundManagerC::GetInstance()->playSoundEffect(SoundManagerC::shotwall,0.5f);
		//Getting a random color
		temp.x = getRangedRandom(PARTICLE_RANDOM_COLOR_MIN_RED,PARTICLE_RANDOM_COLOR_MAX_RED);
		temp.y = getRangedRandom(PARTICLE_RANDOM_COLOR_MIN_GREEN,PARTICLE_RANDOM_COLOR_MAX_GREEN);
		temp.z = getRangedRandom(PARTICLE_RANDOM_COLOR_MIN_BLUE,PARTICLE_RANDOM_COLOR_MAX_BLUE);
		//Creating a new emitter
		int emitterIndex = ParticleManagerC::GetInstance()->newEmitter(ParticleManagerC::CircleEmitter,120,1000,position,velocity,angularVelocity,1.1,50,500, &temp);
		ParticleManagerC::GetInstance()->setParticleBounceOffWalls(emitterIndex,true);
	}

};

void BulletC::render()
{
	if(mEnabled){
		//Push Matrix
		glPushMatrix();

		SpriteManagerC::GetInstance()->drawSprite(mySprite);

		//Pop Matrix
		glPopMatrix();

#if DRAW_COLLISIONS == 1
		DrawCircle(mRadius, mPosition.x, mPosition.y, 1.0f, 0.0f, 0.0f, false);
#endif
	}
}

void BulletC::setPosition(Vector3C newPosition)
{
	 mySprite.setPosition(newPosition.x, newPosition.y, newPosition.z);
}

void BulletC::setPosition(float_t newX, float_t newY)
{
	mPosition.x = newX;
	mPosition.y = newY;
	//mySprite.setPosition(newX, newY, 0.0f);
}

void BulletC::setVelocity(Vector3C newVelocity)
{
	 mVelocity = newVelocity;
}

float_t BulletC::getRadius()
{
	return(mRadius);
}