/*************************************************
**Authors: Kevin Cartrette, Alex Lemke			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary:  Enemy class with modifed render and **
**update to branch from standard enemy behavior.**
*************************************************/

#include "DartEnemy.h"
#include <Windows.h>
#include <stdio.h>
#include <stdlib.h>
#include "stateManager.h"
#include "gl\glu.h"	
#include "ShapeDraw.h"
#include <time.h>
#include "SpriteManager.h"
#include "gamePlayVariables.h"
#include "FieldManager.h"

void DartEnemyC::update(double64_t milliseconds)
{
	if(mEnabled){
		switch(StateManagerC::GetInstance()->getState())
		{
			case StateManagerC::GAMEPLAY:
				FieldManagerC::GetInstance()->applyColor(&mPosition, 10, &enemyColor);
				if (direction == 0)
				{
					mPosition.x += mVelocity.x*milliseconds/1;
					//We don't care about the y velocity.
					mPosition.y += mVelocity.y*milliseconds/1;
					mVelocity.y *= DART_VELOCITY_MODIFER;
				}
				else
				{
					mPosition.y += mVelocity.y*milliseconds/1;
					//We don't care about the x velocity.
					mPosition.x += mVelocity.x*milliseconds/1;
					mVelocity.x *= DART_VELOCITY_MODIFER;
				}

#if(ENEMY_TEXTURES)
				enemySprite.setPosition(mPosition.x, mPosition.y, 0);
#endif
				DoCollisions();
				break;
		}
	}
}

void DartEnemyC::init(float_t initPosX, float_t initPosY, float_t initVelX, float_t initVelY,int32_t id)
{
	mPosition.x = initPosX;
	mPosition.y = initPosY;
	mVelocity.x = initVelX;
	mVelocity.y = initVelY;

	mRadius = ENEMY_RADIUS;
	scoreValue = DART_SCORE;
	health = 1;

	enemyColor.x = 1.0f;
	enemyColor.y = 0.7f;
	enemyColor.z = 0.0f;

#if(ENEMY_TEXTURES)
	enemySprite.setUp(7,0,0,0,64* 2,64 * 2,0,0,0,64,64);
	enemySprite.setCentered(true);
#endif
	direction = randomDirection();
}

void DartEnemyC::setDirection(int32_t newDirection)
{
	direction = newDirection;
}
int32_t DartEnemyC::getDirection()
{
	return direction;
}

void DartEnemyC::render()
{
	if(mEnabled){
		//printf("rendering with dart enemy class!\n");
		glPushMatrix();
		if(mVelocity.getLength() != 0)
		{
			Vector3C straightUp;
			straightUp.setValues(1.0f, 0.0f);

			Vector3C velocityNormalzied;
			velocityNormalzied.setValues(mVelocity.x, mVelocity.y);	
			velocityNormalzied.normalize();
				
			float_t angle = velocityNormalzied.angleBetween(straightUp);

			if(mVelocity.y < 0.0f)
				enemySprite.setRotation(angle * ((-HALF_DEGREES_OF_CIRCLE) / PI_DEFINE) - ONE_FORTH_OF_CIRCLE);
			else	
				enemySprite.setRotation(angle * (HALF_DEGREES_OF_CIRCLE / PI_DEFINE) - ONE_FORTH_OF_CIRCLE);
		}
#if(ENEMY_TEXTURES)
		SpriteManagerC::GetInstance()->drawSprite(enemySprite);
#else
		char8_t red = (char8_t)((mEnemyColor>>16)& 0xFF);
		char8_t green = (char8_t)((mEnemyColor>>8) & 0xFF);
		char8_t blue = (char8_t)((mEnemyColor>>0) & 0xFF);
		bool filledVal;

		filledVal = true;

		DrawCircle(mRadius, mPosition.x,mPosition.y,red,green,blue,filledVal);
#endif

		//SpriteManagerC::GetInstance()->drawSprite();

		//Pop Matrix
		glPopMatrix();
	}
}

int32_t DartEnemyC::randomDirection()
{
	return (rand()%2);
}