/*************************************************
**Authors: Kevin Cartrette, Alex Lemke			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary:  Enemy class with modifed render and **
**update to branch from standard enemy behavior.**
*************************************************/

#include "DodgerEnemy.h"
#include <Windows.h>
#include <stdio.h>
#include <stdlib.h>
#include "stateManager.h"
#include "gl\glu.h"	
#include "ShapeDraw.h"
#include <time.h>
#include "SpriteManager.h"
#include "InputManager.h"
#include "random.h"
#include "gamePlayVariables.h"
#include "FieldManager.h"

void DodgerEnemyC::init(float_t initPosX, float_t initPosY, float_t initVelX, float_t initVelY, int32_t id)
{
	mPosition.x = initPosX;
	mPosition.y = initPosY;
	mVelocity.x = initVelX;
	mVelocity.y = initVelY;

	mRadius = ENEMY_RADIUS;
	scoreValue = DODGER_SCORE;
	health = 1;

	enemyColor.x = 0.0f;
	enemyColor.y = 1.0f;
	enemyColor.z = 0.0f;


#if(ENEMY_TEXTURES)
	enemySprite.setUp(8,0,0,0,64 * 2,64 * 2,0,0,0,64,64);
	enemySprite.setCentered(true);
#endif
}

void DodgerEnemyC::update(double64_t milliseconds)
{
	if(mEnabled){
		switch(StateManagerC::GetInstance()->getState())
		{
			case StateManagerC::GAMEPLAY:
				FieldManagerC::GetInstance()->applyColor(&mPosition, 10, &enemyColor);
				Vector3C *playerPosition = getPlayerPosition();

				Vector3C *myPosition = getPosition();
				float_t differenceX = playerPosition->x - mPosition.x;
				float_t differenceY = playerPosition->y - mPosition.y;

				float_t value = myPosition->dotProduct(InputManagerC::GetInstance()->returnMouseGamePosition());

				//Check dotproduct of firing direction vector, if firing then move based on that.

				Vector3C velocityChange;
				if (value < 0 && InputManagerC::GetInstance()->isLeftMouseButtonDown())
				{
					velocityChange.x -= differenceX*milliseconds/getRangedRandom(900,1200);
					velocityChange.y -= differenceY*milliseconds/getRangedRandom(300,600);
				}
				else
				{
					velocityChange.x += differenceX*milliseconds/getRangedRandom(900,1200);
					velocityChange.y += differenceY*milliseconds/getRangedRandom(900,1200);
				}
				
				velocityChange.scaleVector(0.15f);

				//Friction
				mVelocity.scaleVector(0.9f);

				//Increase velocity
				mVelocity.add2DQuickVector(velocityChange);

				//Increase position
				mPosition.add2DQuickVector(mVelocity);

#if(ENEMY_TEXTURES)
				enemySprite.setPosition(mPosition.x, mPosition.y, 0);
#endif
				DoCollisions();
				break;
		}
	}
}

void DodgerEnemyC::render()
{
	if(mEnabled){
		//printf("rendering with dart enemy class!\n");
		glPushMatrix();
#if(ENEMY_TEXTURES)
		SpriteManagerC::GetInstance()->drawSprite(enemySprite);
#else
		char8_t red = (char8_t)((mEnemyColor>>16)& 0xFF);
		char8_t green = (char8_t)((mEnemyColor>>8) & 0xFF);
		char8_t blue = (char8_t)((mEnemyColor>>0) & 0xFF);
		bool filledVal;

		filledVal = true;

		DrawCircle(mRadius, mPosition.x,mPosition.y,red,green,blue,filledVal);
#endif

		//SpriteManagerC::GetInstance()->drawSprite();

		//Pop Matrix
		glPopMatrix();
	}
}