/*************************************************
**Authors: Kevin Cartrette, Alex Lemke			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary:  Enemy manager class which processes **
**everything to do with enemies.				**
*************************************************/

#define ENEMY_MANAGER_CPP
#include <assert.h>
#include <windows.h>										
#include <stdio.h>											
#include <stdarg.h>	
#include <math.h>	
#include "gl\gl.h"
#include "gl\glu.h"											
#include "gl\glut.h"
#include "baseTypes.h"
#include "openglframework.h"	
#include "collinfo.h"
#include "object.h"
#include "gamePlayVariables.h"
#include "enemy.h"
#include "DartEnemy.h"
#include "ChaserEnemy.h"
#include "SpinnerEnemy.h"
#include "DodgerEnemy.h"
#include "BlackHoleEnemy.h"
#include "ProtonEnemy.h"
#include "GateEnemy.h"
#include "EnemyManager.h"
#include "SoundManager.h"
#include "random.h"
#include "SpriteManager.h"
#include "FieldManager.h"
#include "PlayerManager.h"
#include "ParticleManager.h"
#include "PickUpManager.h"
#include "CameraManager.h"
#include "stateManager.h"

EnemyManagerC* EnemyManagerC::sInstance=NULL;

EnemyManagerC *EnemyManagerC::CreateInstance()
{
	assert(sInstance==NULL);
	sInstance = new EnemyManagerC();
	return sInstance;
}

void EnemyManagerC::init()
{
	currentTime = 0;
	timeBetweenUpdates = ENEMY_TIME_BETWEEN_UPDATES;
	timeSinceLastSpawn = 0;
	timeBetweenSpawns = ENEMY_TIME_BETWEEN_SPAWNS;
	minEnemies = MIN_ENEMIES;
	maxEnemies = MAX_ENEMIES;

}

void EnemyManagerC::updateEnemies(double64_t milliseconds)
{
	totalTimePassed += milliseconds;
	switch(StateManagerC::GetInstance()->getGameMode())
	{
		case StateManagerC::SURVIVAL_MODE:
			currentTime += milliseconds;
			//printf("current time %lf\n", currentTime);
			if(currentTime >= timeBetweenUpdates)
			{
				timeSinceLastSpawn += currentTime;
				updateEnemyList(currentTime);
				checkPlayerCollisions(0);
				if(timeSinceLastSpawn >= timeBetweenSpawns)
				{
					handleSpawns(StateManagerC::GetInstance()->getGameMode());
					timeSinceLastSpawn = 0;
				}
				currentTime = 0;
			}
			break;
		case StateManagerC::WAVE_MODE:
			currentTime += milliseconds;
			//printf("current time %lf\n", currentTime);
			if(currentTime >= timeBetweenUpdates)
			{
				timeSinceLastSpawn += currentTime;
				updateEnemyList(currentTime);
				checkPlayerCollisions(0);
				if(timeSinceLastSpawn >= timeBetweenSpawns)
				{
					handleSpawns(StateManagerC::GetInstance()->getGameMode());
					timeSinceLastSpawn = 0;
				}
				currentTime = 0;
			}
			break;
		case StateManagerC::WORM_HOLE_MODE:
			currentTime += milliseconds;
			//printf("current time %lf\n", currentTime);
			if(currentTime >= timeBetweenUpdates)
			{
				timeSinceLastSpawn += currentTime;
				updateEnemyList(currentTime);
				checkPlayerCollisions(0);
				if(timeSinceLastSpawn >= timeBetweenSpawns)
				{
					handleSpawns(StateManagerC::GetInstance()->getGameMode());
					timeSinceLastSpawn = 0;
				}
				currentTime = 0;
			}
			break;
		case StateManagerC::PACIFISM_MODE:
			currentTime += milliseconds;
			//printf("current time %lf\n", currentTime);
			if(currentTime >= timeBetweenUpdates)
			{
				timeSinceLastSpawn += currentTime;
				updateEnemyList(currentTime);
				checkPlayerCollisions(0);
				if(timeSinceLastSpawn >= timeBetweenSpawns)
				{
					handleSpawns(StateManagerC::GetInstance()->getGameMode());
					timeSinceLastSpawn = 0;
				}
				currentTime = 0;
			}
			break;
	}
}

void EnemyManagerC::handleSpawns(int32_t currentGameMode)
{
	Vector3C position;
	Vector3C velocity;
	position.x = 0;
	position.y = 0;
	velocity.x = 1;
	velocity.y = 1;
	double64_t playerScore = (double64_t)PlayerManagerC::GetInstance()->getPlayerPtr(0)->getScore();
	int32_t numberEnemiesSpawn = totalTimePassed / (minEnemies + maxEnemies);
	numberEnemiesSpawn = 20;
	switch(currentGameMode)
	{
		case StateManagerC::SURVIVAL_MODE:
			if((int32_t)enemyList.size() < maxEnemies)
			{
				newWave(numberEnemiesSpawn);
			}
			break;
		case StateManagerC::WAVE_MODE:
			if((int32_t)enemyList.size() < maxEnemies)
			{
				position.x = getRangedRandom(-ENEMY_RANDOM_SPAWN_RANGE, ENEMY_RANDOM_SPAWN_RANGE);
				position.y = getRangedRandom(-ENEMY_RANDOM_SPAWN_RANGE, ENEMY_RANDOM_SPAWN_RANGE);
				velocity.x = getRangedRandom(0.5f,1.0f);
				velocity.y = getRangedRandom(0.5f,1.0f);

				spawnDartWave(position,velocity,numberEnemiesSpawn,getRangedRandom(0,2),DART_ENEMY_OFFSET);
				newWave(DART_MODE_EXTRA_ENEMIES);
			}
			break;
		case StateManagerC::WORM_HOLE_MODE:
			if((int32_t)enemyList.size() < maxEnemies)
			{
				spawnBlackHoleWave(position,velocity,numberEnemiesSpawn);
				newWave(BLACK_HOLE_MODE_EXTRA_ENEMIES);
			}
			break;
		case StateManagerC::PACIFISM_MODE:
			if((int32_t)enemyList.size() < maxEnemies)
			{
				spawnGateWave(position,velocity,GATE_MODE_GATES_PER_SPAWN);
				spawnAllFourCorners(CHASER_ENEMY,numberEnemiesSpawn);
			}
			break;
	}
}

void EnemyManagerC::newWave(int32_t numberToSpawn)
{
	for(int32_t i = 0; i < numberToSpawn; i++)
	{
		spawnRandomEnemy();
	}
}

void EnemyManagerC::updateEnemyList(double64_t currentTime)
{
	//Regular Updates - may be crashing
	for (enemyIterator = enemyList.begin(); enemyIterator != enemyList.end();)
	{
		if(*enemyIterator){
			(*enemyIterator)->update(currentTime);
			if((*enemyIterator) && (enemyIterator != enemyList.end()))
				enemyIterator++;
		}
		else{
			break;
		}
	}
}

void EnemyManagerC::checkPlayerCollisions(int32_t playerNumber)
{
	for (enemyIterator = enemyList.begin(); enemyIterator != enemyList.end();)
	{
		if((*enemyIterator)->getEnabled())
		{
			if((*enemyIterator)->checkIfBulletsCollide(playerNumber))
			{
				//Decrease the enemies health
				(*enemyIterator)->decreaseHealth();
				if((*enemyIterator)->getHealth() <= 0)
				{
					killEnemy(playerNumber);		
				}
			}
			else
			{
				enemyIterator++;
			}
		}
		else
		{//The enemy is off so remove them!
			delete (*enemyIterator);
			enemyIterator = enemyList.erase(enemyIterator);
		}
	}
}

bool EnemyManagerC::checkEnemyRadialCollision(double64_t radius, Vector3C position, bool killHitEnemy)
{
	bool hitHappen = false;
	for (enemyIterator = enemyList.begin(); enemyIterator != enemyList.end();)
	{
		if((*enemyIterator)->getEnabled())
		{
				//checkIfPlayerCollide(radius, position)
				if((*enemyIterator)->checkIfCollideRadial(radius, position))
				//if((*enemyIterator)->checkIfPlayerPositionCollide(radius, position))
				{
					if(position.distance(*(*enemyIterator)->getPosition()) != 0)
					{
						hitHappen = true;
						//Early out less checks.
						if(hitHappen == true && killHitEnemy == false)
						{
							return true;
						}

						if(killHitEnemy == true)
						{
							int playerNumber = 0; //Temp hack for player number.
							killEnemy(playerNumber);
						}
					}
					else
					{
						//Go to the next enemy
						enemyIterator++;
					}
				}
				else
				{
					//Go to the next enemy
					enemyIterator++;
				}
			}
			else
			{//The enemy is off so remove them!
				delete (*enemyIterator);
				enemyIterator = enemyList.erase(enemyIterator);
			}
	}
    return hitHappen;
}

bool EnemyManagerC::checkEnemyRadialCollision(double64_t radius, int32_t playerNumber, bool killHitEnemy)
{
	bool hitHappen = false;
	for (enemyIterator = enemyList.begin(); enemyIterator != enemyList.end();)
	{
		if((*enemyIterator)->getEnabled())
		{
				Vector3C position = *PlayerManagerC::GetInstance()->getPlayerPtr( playerNumber)->getPosition();

				//checkIfPlayerCollide(radius, position)
				//((*enemyIterator)->checkIfCollideRadial(radius, position))
				if((*enemyIterator)->checkIfPlayerPositionCollide(radius, playerNumber))
				{
					if(position.distance(*(*enemyIterator)->getPosition()) != 0){

						hitHappen = true;
						//Early out less checks.
						if(hitHappen == true && killHitEnemy == false)
						{
							return true;
						}

						if(killHitEnemy == true)
						{
							int playerNumber = 0; //Temp hack for player number.
							killEnemy(playerNumber);
						}
					}
					else
					{
						//Go to the next enemy
						enemyIterator++;
					}
				}
				else
				{
					//Go to the next enemy
					enemyIterator++;
				}
			}
			else
			{//The enemy is off so remove them!
				delete (*enemyIterator);
				enemyIterator = enemyList.erase(enemyIterator);
			}
	}
    return hitHappen;
}

EnemyC *EnemyManagerC::getEnemyPtr(int32_t index)
{
	//return enemyPtrs[index];
	return(enemyList[index]);
}

void EnemyManagerC::renderEnemys()
{
	//Push Matrix
	glPushMatrix();

	for (enemyIterator = enemyList.begin(); enemyIterator != enemyList.end(); enemyIterator++)
	{
		if(CameraManagerC::GetInstance()->checkIfInViewport(*((*enemyIterator)->getPosition()))) //Only draw if it's in the current camera view
			(*enemyIterator)->render();
    }

	//Pop Matrix
	glPopMatrix();
}

void EnemyManagerC::shutdown()
{
	for (enemyIterator = enemyList.begin(); enemyIterator != enemyList.end();)
    {
		delete(*enemyIterator);
		enemyIterator = enemyList.erase(enemyIterator);
	}
}

void EnemyManagerC::reset()
{
	for (enemyIterator = enemyList.begin(); enemyIterator != enemyList.end();)
    {
		delete(*enemyIterator);
		enemyIterator = enemyList.erase(enemyIterator);
	}
}

void EnemyManagerC::removeEnemy(vector<EnemyC*>::iterator enemyIndex)
{
	delete(*enemyIndex);
	enemyIterator = enemyList.erase(enemyIndex);
}

//Create Protons
void EnemyManagerC::createProtons(Vector3C position)
{
	for(int32_t j = 0; j < NUMBER_PROTONS_ON_BLACK_HOLE_DEATH; j ++)
	{
		enemyList.push_back(new ProtonEnemyC);
		int32_t enemySpot = enemyList.size() - 1;
		enemyList[enemySpot]->init(position.x, position.y, getRangedRandom(-1.0f,1.0f),getRangedRandom(-1.0f,1.0f), j);
		enemyList[enemySpot]->setRandomColor();
		enemyList[enemySpot]->enable();
		enemyList[enemySpot]->setID(enemySpot);
	}
}

//Kill Enemy Function - kills the enemy that enemyIterator is currently pointing at
void EnemyManagerC::killEnemy(int32_t playerNumber)
{
	//Start enemy death events
	enemyDeathEvent(playerNumber, *(*enemyIterator)->getPosition(), (*enemyIterator)->getScoreValue());
	
	//Remove the Enemy
	(*enemyIterator)->disable();
	delete (*enemyIterator);
	enemyIterator = enemyList.erase(enemyIterator);
}

void EnemyManagerC::spawnDartWave(Vector3C position, Vector3C velocity, int32_t numberInSpawn, int32_t direction, int32_t offset)
{
	int32_t enemyNumber;
	for(int32_t i = 0; i < numberInSpawn; i++)
	{
		if(direction == 0)
		{
			position.y -= offset;
		}
		else if(direction == 1)
		{
			position.x -= offset;
		}
	
		enemyNumber = spawnEnemy(DART_ENEMY,position, velocity);
		enemyList[enemyNumber]->setDirection(direction);
	}
}

void EnemyManagerC::spawnAllFourCorners(int32_t enemyNumber, int32_t numberOfEnemies)
{
	int randomCorner;
	Vector3C position;
	position.x = ENEMY_RANDOM_SPAWN_RANGE;
	position.y = ENEMY_RANDOM_SPAWN_RANGE;
	Vector3C velocity;
	velocity.setValues(1.0f,1.0f);
	for(int32_t i = 0; i < numberOfEnemies; i++)
	{
		velocity.x = getRangedRandom(0.0f,1.0f);
		velocity.y = getRangedRandom(0.0f,1.0f);
		position.x = ENEMY_RANDOM_SPAWN_RANGE;
		position.y = ENEMY_RANDOM_SPAWN_RANGE;
		randomCorner = getRangedRandom(0,4);
		switch(randomCorner)
		{
			case 1:
				position.x = -ENEMY_RANDOM_SPAWN_RANGE;
				break;
			case 2:
				position.y = -ENEMY_RANDOM_SPAWN_RANGE;
				break;
			case 3:
				position.x = -ENEMY_RANDOM_SPAWN_RANGE;
				position.y = -ENEMY_RANDOM_SPAWN_RANGE;
				break;
			default:
				break;
		}
		
		spawnEnemy(CHASER_ENEMY, position, velocity);
	}
}

//Spawn a wave of gate enemies.
void EnemyManagerC::spawnGateWave(Vector3C position, Vector3C velocity, int32_t numberInSpawn)
{
	for(int32_t i = 0; i < numberInSpawn; i++)
	{
		position.x = getRangedRandom(-ENEMY_RANDOM_SPAWN_RANGE, ENEMY_RANDOM_SPAWN_RANGE);
		position.y = getRangedRandom(-ENEMY_RANDOM_SPAWN_RANGE, ENEMY_RANDOM_SPAWN_RANGE);
		spawnEnemy(GATE_ENEMY,position, velocity);
	}
}
void EnemyManagerC::spawnBlackHoleWave(Vector3C position, Vector3C velocity, int32_t numberInSpawn)
{
	for(int32_t i = 0; i < numberInSpawn; i++)
	{
		position.x = getRangedRandom(-ENEMY_RANDOM_SPAWN_RANGE, ENEMY_RANDOM_SPAWN_RANGE);
		position.y = getRangedRandom(-ENEMY_RANDOM_SPAWN_RANGE, ENEMY_RANDOM_SPAWN_RANGE);
		spawnEnemy(BLACK_HOLE_ENEMY,position, velocity);
	}
}

int32_t EnemyManagerC::spawnEnemy(int32_t enemyNumber, Vector3C position, Vector3C velocity)
{
	switch(enemyNumber)
	{
		case DART_ENEMY:
			enemyList.push_back(new DartEnemyC);
			break;
		case SPINNER_ENEMY:
			enemyList.push_back(new SpinnerEnemyC);
			break;
		case CHASER_ENEMY:
			enemyList.push_back(new ChaserEnemyC);
			break;
		case DODGER_ENEMY:
			enemyList.push_back(new DodgerEnemyC);
			break;
		case BLACK_HOLE_ENEMY:
			enemyList.push_back(new BlackHoleEnemyC);
			break;
		case GATE_ENEMY:
			enemyList.push_back(new GateEnemyC);
			break;
	}

	int32_t enemySpot = enemyList.size() - 1;
	enemyList[enemySpot]->init(position.x, position.y, velocity.x, velocity.y, (enemyList.size() - 1));
	enemyList[enemySpot]->setRandomColor();
	enemyList[enemySpot]->enable();
	enemyList[enemySpot]->setID(enemySpot);
	return enemySpot;
}

void EnemyManagerC::spawnRandomEnemy()
{
	int32_t enemy = getRangedRandom(INVALID_ENEMY+1, MAX_ENEMY);

	switch(enemy)
	{
		case DART_ENEMY:
			enemyList.push_back(new DartEnemyC);
			break;
		case SPINNER_ENEMY:
			enemyList.push_back(new SpinnerEnemyC);
			break;
		case CHASER_ENEMY:
			enemyList.push_back(new ChaserEnemyC);
			break;
		case DODGER_ENEMY:
			enemyList.push_back(new DodgerEnemyC);
			break;
		case BLACK_HOLE_ENEMY:
			enemyList.push_back(new BlackHoleEnemyC);
			break;
		case GATE_ENEMY:
			enemyList.push_back(new GateEnemyC);
			break;
	}

	int32_t enemySpot = enemyList.size() - 1;
	enemyList[enemySpot]->init(getRangedRandom(-ENEMY_RANDOM_SPAWN_RANGE,ENEMY_RANDOM_SPAWN_RANGE), getRangedRandom(-ENEMY_RANDOM_SPAWN_RANGE,ENEMY_RANDOM_SPAWN_RANGE),
		getRangedRandom(-1.0f,1.0f),getRangedRandom(-1.0f,1.0f), (enemyList.size() - 1));
	enemyList[enemySpot]->setRandomColor();
	enemyList[enemySpot]->enable();
	enemyList[enemySpot]->setID(enemySpot);
}

//Enemy Death Event
void EnemyManagerC::enemyDeathEvent(int32_t playerNumber, Vector3C eventPosition, int32_t scoreAmount)
{
	//Creating Particles.
	Vector3C position(eventPosition.x, eventPosition.y, 0.0f);
	Vector3C velocity(CIRCLE_EMITTER_VELOCITY,CIRCLE_EMITTER_VELOCITY,0);
	Vector3C angularVelocity(CIRCLE_EMITTER_ANGULAR_VELOCITY, CIRCLE_EMITTER_ANGULAR_VELOCITY,0);
	Vector3C particleColor;

	//Getting a random color
	temp.x = getRangedRandom(PARTICLE_RANDOM_COLOR_MIN_RED,PARTICLE_RANDOM_COLOR_MAX_RED);
	temp.y = getRangedRandom(PARTICLE_RANDOM_COLOR_MIN_GREEN,PARTICLE_RANDOM_COLOR_MAX_GREEN);
	temp.z = getRangedRandom(PARTICLE_RANDOM_COLOR_MIN_BLUE,PARTICLE_RANDOM_COLOR_MAX_BLUE);

	ParticleManagerC::GetInstance()->newEmitter(ParticleManagerC::CircleEmitter,CIRCLE_EMITTER_TIME_UPDATE,CIRCLE_EMITTER_TIME_TO_LIVE,position,velocity,angularVelocity,1.1,CIRCLE_EMITTER_MIN_PARTICLES,CIRCLE_EMITTER_MAX_PARTICLES, (&temp));

	//Creating Multiplier PickUps.
	int32_t numberOfGeneratedPickUps = (int32_t)getRangedRandom(1, MAX_NUMBER_PICKUPS);
	Vector3C initalPickUpPosition(position.x, position.y, 0.0f);
	for(int32_t i = 0; i < numberOfGeneratedPickUps; i++)
	{
		Vector3C pickUpVelocity(getRangedRandom(-1, 1), getRangedRandom(-1, 1));
		pickUpVelocity.normalize();
		pickUpVelocity.scaleVector(1.9f);
		PickUpManagerC::GetInstance()->createNewPickup(initalPickUpPosition, pickUpVelocity); 
	}

	

	//Playing Explosion Sound.
	SoundManagerC::GetInstance()->playSoundEffect(SoundManagerC::Explo1,1.0f);
				
	//Pushing the Grid.
	FieldManagerC::GetInstance()->applyPush((&eventPosition), GRID_RADIUS_APPLIED_ON_MENU, GRID_FORCE_APPLIED_ON_ENEMY_DEATH, &temp);
				
	//Increase  the player's score.
	PlayerManagerC::GetInstance()->getPlayerPtr(playerNumber)->increaseScore(scoreAmount);
}