/*************************************************
**Authors: Kevin Cartrette, Alex Lemke			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary:  Enemy manager class which processes **
**everything to do with enemies.				**
*************************************************/

#include <vector>
#include <stdlib.h>
#include "enemy.h"

using namespace std;

/*************************************************
**Summary: Enemy manger which creates and		**
**controls all interactions with enemies.		**
**Authors: Kevin Cartrette, Alex Lemke			**
*************************************************/
class EnemyManagerC
{
public:
	enum
	{
		INVALID_ENEMY,
		DART_ENEMY,
		SPINNER_ENEMY,
		CHASER_ENEMY,
		DODGER_ENEMY,
		BLACK_HOLE_ENEMY,
		GATE_ENEMY,
		MAX_ENEMY,
	};

	static EnemyManagerC	*CreateInstance();
	static EnemyManagerC	*GetInstance() {return sInstance;};
	~EnemyManagerC(){};

	//Function prototypes.
	void init();
	void reset();
	void shutdown();
	void randomEnemy();
	void renderEnemys();
	EnemyC*	getEnemyPtr(int32_t index);
	void newWave(int32_t numberToSpawn);
	void handleSpawns(int32_t currentGameMode);
	void updateEnemies(double64_t milliseconds);
	void updateEnemyList(double64_t currentTime);
	void checkPlayerCollisions(int32_t playerNumber);
	int32_t	getNumEnemies() {return enemyList.size();};
	void removeEnemy(vector<EnemyC*>::iterator enemyIndex);
	void spawnAllFourCorners(int32_t enemyNumber, int32_t numberOfEnemies);
	void spawnGateWave(Vector3C position, Vector3C velocity, int32_t numberInSpawn);
	int32_t spawnEnemy(int32_t enemyNumber, Vector3C position, Vector3C velocity);
	
	bool checkEnemyRadialCollision(double64_t radius, Vector3C position, bool killHitEnemy);
	bool checkEnemyRadialCollision(double64_t radius, int32_t playerNumber, bool killHitEnemy);


	void spawnDartWave(Vector3C position, Vector3C velocity, int32_t numberInSpawn, int32_t direction, int32_t offset);
	void spawnBlackHoleWave(Vector3C position, Vector3C velocity, int32_t numberInSpawns);
	//Create Protons
	void createProtons(Vector3C position);
	//Kill Enemy Function - kills the enemy that enemyIterator is currently pointing at
	void killEnemy(int32_t playerNumber);
	//Spawn a random enemy
	void spawnRandomEnemy();
	//Enemy Death Event
	void enemyDeathEvent(int32_t playerNumber, Vector3C eventPosition, int32_t scoreAmount);
private:
	//Vector of enemy class objects.
	vector<EnemyC*> enemyList;
	vector<EnemyC*>::iterator enemyIterator;
	//Time variables to deal with update time events.
	double64_t currentTime;
	double64_t timeBetweenUpdates;
	double64_t timeSinceLastSpawn;
	double64_t timeBetweenSpawns;
	//Min and max enemies.
	int32_t minEnemies;
	int32_t maxEnemies;
	double64_t totalTimePassed;
	static EnemyManagerC *sInstance;
	EnemyManagerC(){};

	//Temporary Vector 3 used for almost anything and everything
	Vector3C temp;
};