/*************************************************
**Authors: Kevin Cartrette, Alex Lemke			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary: GUI Manager which handles displaying **
**text and textures to the screen.				**
*************************************************/

#define FIELD_MANAGER_CPP
#include <windows.h>											
#include <stdio.h>											
#include <stdarg.h>												
#include <math.h>												
#include "gl\gl.h"												
#include "gl\glu.h"												
#include "gl\glut.h"
#include "baseTypes.h"
#include "openglframework.h"	
#include "collinfo.h"
#include "object.h"
#include "GUIManager.h"
#include "EnemyManager.h"
#include "ParticleManager.h"
#include "PlayerManager.h"
#include "player.h"
#include "glext.h"
#include "SpriteManager.h"
#include "gamePlayVariables.h"

GUIManagerC* GUIManagerC::sInstance=NULL;

GUIManagerC *GUIManagerC::CreateInstance()
{
	if(sInstance!=NULL)return sInstance;
	else
	sInstance = new GUIManagerC();
	return sInstance;
}

void GUIManagerC::init()
{
	//Frames Per Second.
	frameCount = 0;
	fps = 0;
	currentTime = 0;
	previousTime = 0;

	//Fonts.
	if (!glFontCreate(&myFont, "Assets\\Fonts\\neuropol", 20))
		exit(1);

	//Icon Sprites.
	lifeIconSprite.setUp(10, -400, 1020, 0, 64 * 2, 64 * 2, 0, 0, 64, 64, 64);
	lifeIconSprite.setRotation(180);
	lifeIconSprite.setCentered(true);

	bombIconSprite.setUp(10, 400, 1020, 0, 64 * 2, 64 * 2, 0, 0, 0, 64, 64);
	bombIconSprite.setCentered(true);
}

void GUIManagerC::renderGUI()
{
	drawDebugInformation();
	drawGUI();
}

void GUIManagerC::shutdown()
{
}

void GUIManagerC::calculateFPS()
{
    //Increase frame count.
    frameCount++;

    //Get the number of milliseconds since glutInit called.
    //(or first call to glutGet(GLUT ELAPSED TIME)).
    currentTime = glutGet(GLUT_ELAPSED_TIME);

    //Calculate time passed
    int32_t timeInterval = currentTime - previousTime;

    if(timeInterval > 1000)
    {
        //Calculate the number of frames per second.
        fps = frameCount / (timeInterval / 1000.0f);

        //Set time.
        previousTime = currentTime;

        //Reset frame count.
        frameCount = 0;
    }
}

void GUIManagerC::drawDebugInformation()
{
#if DEBUG == 1

    //  Load the identity matrix so that FPS string being drawn
    //  won't get animates
	glLoadIdentity ();
	char8_t debugString[100];
	char8_t numberOfEnemiesString[100];
	char8_t numberOfParticlesString[100];
	glColor3f(0.0f,1.0f,0.0f);

	glEnable(GL_TEXTURE_2D);

	glEnable(GL_BLEND);

	sprintf_s(debugString,"FPS: %4.2f \n", fps);
	sprintf_s(numberOfEnemiesString,"Number Of Enemies: %d \n", EnemyManagerC::GetInstance()->getNumEnemies());
	sprintf(numberOfParticlesString,"Number Of Particles: %d \n", ParticleManagerC::GetInstance()->getTotalNumberOfParticles());
	
	//FPS Counter.
	glPushMatrix();
	glFontBegin(&myFont);
	glTranslatef(-1200, 600, 0);
	glScalef(20.0, 20.0, 0.0);
	glFontTextOut(debugString, 10, 10, 0);
	glFontEnd();
	glPopMatrix();

	//Number Of Enemies.
	glPushMatrix();
	glFontBegin(&myFont);
	glTranslatef(-1200, 550, 0);
	glScalef(20.0, 20.0, 0.0);
	glFontTextOut(numberOfEnemiesString, 10, 10, 0);
	glFontEnd();
	glPopMatrix();

	//Number Of Particles.
	glPushMatrix();
	glFontBegin(&myFont);
	glTranslatef(-1200, 500, 0);
	glScalef(20.0, 20.0, 0.0);
	glFontTextOut(numberOfParticlesString, 10, 10, 0);
	glFontEnd();
	glPopMatrix();
#endif
}

void GUIManagerC::startDefaultFont(int32_t xLocation, int32_t yLocation)
{
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glPushMatrix();
	glFontBegin(&myFont);
	glTranslatef(xLocation, yLocation, 0);
	glScalef(20.0, 20.0, 0.0);
}

void GUIManagerC::endDefaultFont()
{
	glFontEnd();
	glPopMatrix();
}

void GUIManagerC::updateGUI(double64_t milliseconds)
{
	//Frames Per Second.
	calculateFPS();
}

//*********************************************************
//glFont routines by Brad Fish
//*********************************************************
int GUIManagerC::glFontCreate (GLFONT *Font, char8_t *FileName, int32_t Tex)
{
	FILE *Input;
	char8_t *TexBytes;
	int32_t Num;

	//Open font file
	if ((Input = fopen(FileName, "rb")) == NULL)
		return FALSE;

	//Read glFont structure
	fread(Font, sizeof(GLFONT), 1, Input);

	//Save texture number
	Font->Tex = Tex;

	//Get number of characters
	Num = Font->IntEnd - Font->IntStart + 1;
	
	//Allocate memory for characters
	if ((Font->Char = (GLFONTCHAR *)malloc(
		sizeof(GLFONTCHAR) * Num)) == NULL)
		return FALSE;
	
	//Read glFont characters
	fread(Font->Char, sizeof(GLFONTCHAR), Num, Input);

	//Get texture size
	Num = Font->TexWidth * Font->TexHeight * DOUBLE;
	
	//Allocate memory for texture data
	if ((TexBytes = (char8_t *)malloc(Num)) == NULL)
		return FALSE;
	
	//Read texture data
	fread(TexBytes, sizeof(char8_t), Num, Input);

	//Set texture attributes
	glBindTexture(GL_TEXTURE_2D, Font->Tex);  
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S,
		GL_CLAMP);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,
		GL_CLAMP);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER,
		GL_LINEAR); 
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
		GL_LINEAR);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE,
		GL_MODULATE);  
	
	//Create texture
	glTexImage2D(GL_TEXTURE_2D, 0, 2, Font->TexWidth,
		Font->TexHeight, 0, GL_LUMINANCE_ALPHA, 
		GL_UNSIGNED_BYTE, (void *)TexBytes);

	//Clean up
	free(TexBytes);
	fclose(Input);

	//Return pointer to new font
	return TRUE;
}
void GUIManagerC::glFontDestroy (GLFONT *Font)
{
	//Free character memory
	free(Font->Char);
}
void GUIManagerC::glFontBegin (GLFONT *Font)
{
	//Save pointer to font structure
	if (Font->Char != NULL)
		glFont = Font;
	else
		glFont = NULL;
	
	//Bind to font texture
	glBindTexture(GL_TEXTURE_2D, Font->Tex);
}
void GUIManagerC::glFontEnd (void)
{
	//Font no longer current
	glFont = NULL;
}
void GUIManagerC::setFontColor(float_t red, float_t green, float_t blue)
{
	glColor3f(red,green,blue);
}

void GUIManagerC::glFontTextOut (char8_t *String, float_t x, float_t y, float_t z)
{
	int32_t Length, i;
	GLFONTCHAR *Char;

	//Return if we don't have a valid glFont 
	if (glFont == NULL)
		return;
	
	//Get length of string
	Length = strlen(String);
	
	//Begin rendering quads
	glBegin(GL_QUADS);
	
	//Loop through characters
	for (i = 0; i < Length; i++)
	{
		//Get pointer to glFont character
		Char = &glFont->Char[(int32_t)String[i] -
			glFont->IntStart];
		
		//Specify vertices and texture coordinates
		glTexCoord2f(Char->tx1, Char->ty1);
		glVertex3f(x, y, z);
		glTexCoord2f(Char->tx1, Char->ty2);
		glVertex3f(x, y - Char->dy, z);
		glTexCoord2f(Char->tx2, Char->ty2);
		glVertex3f(x + Char->dx, y - Char->dy, z);
		glTexCoord2f(Char->tx2, Char->ty1);
		glVertex3f(x + Char->dx, y, z);
	
		//Move to next character
		x += Char->dx;
	}

	//Stop rendering quads
	glEnd();
}

void GUIManagerC::drawGUI()
{
	glColor3f(0.0f,1.0f,0.0f);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);

	char8_t scoreString[100];
	sprintf_s(scoreString, "Score: %d \n", PlayerManagerC::GetInstance()->getPlayerPtr(0)->getScore());

	char8_t multiplierString[100];
	sprintf_s(multiplierString, "x: %d \n", PlayerManagerC::GetInstance()->getPlayerPtr(0)->getMultiplier());
	
	char8_t lifeString[100];
	sprintf_s(lifeString, "x: %d \n", PlayerManagerC::GetInstance()->getPlayerPtr(0)->getLives());

	char8_t bombString[100];
	sprintf_s(bombString, "x: %d \n", PlayerManagerC::GetInstance()->getPlayerPtr(0)->getNumberBombs());

	glLoadIdentity();

	glPushMatrix();
	gluPerspective(0.45f, 1, 0.01f, 1000.0f);

	glPushMatrix();
	glTranslatef(0, 0, -300);

	//Score
	glPushMatrix();
	glFontBegin(&myFont);
	glTranslatef(-1400, 800, 0);
	//glRotatef(10, 1, 1, 0);
	//glRotatef(45.0f, 0.0f, 1.0f, 0.0f);
	glScalef(30.0f, 30.0f, 0.0f);
	glFontTextOut(scoreString, 10, 10, 0);
	glFontEnd();
	glPopMatrix();

	glTranslatef(0, 0, -10);

	//Multiplier
	glPushMatrix();
	glFontBegin(&myFont);
	glTranslatef(-350, 750, 0);
	glScalef(30.0, 30.0, 0.0);
	glFontTextOut(multiplierString, 10, 10, 0);
	glFontEnd();
	glPopMatrix();

	//Lives
	glPushMatrix();
	glFontBegin(&myFont);
	glTranslatef(-650, 750, 0);
	glScalef(30.0f, 30.0f, 0.0f);
	glFontTextOut(lifeString, 10, 10, 0);
	glFontEnd();
	glPopMatrix();

	//Bombs
	glPushMatrix();
	glFontBegin(&myFont);
	glTranslatef(150, 750, 0);
	glScalef(30.0f, 30.0f, 0.0f);
	glFontTextOut(bombString, 10, 10, 0);
	glFontEnd();
	glPopMatrix();

	//Icons
	SpriteManagerC::GetInstance()->drawSprite(lifeIconSprite);
	SpriteManagerC::GetInstance()->drawSprite(bombIconSprite);

	glPopMatrix();
	glMatrixMode (GL_MODELVIEW);
	glPopMatrix();
}

//Return the FPS
float_t GUIManagerC::returnFPS()
{
	return(fps);
}