/*************************************************
**Authors: Kevin Cartrette, Alex Lemke			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary: GUI Manager which handles displaying **
**text and textures to the screen.				**
*************************************************/

#ifndef GUIMANAGER_H
#define GUIMANAGER_H

#include "Sprite.h"
#include "baseTypes.h"

/*************************************************
**Summary: GUI manager which out puts GUI.		**
**Authors: Kevin Cartrette, Alex Lemke			**
*************************************************/
class GUIManagerC
{
public:
	//glFont character structure.
	typedef struct
	{
		float_t dx, dy;
		float_t tx1, ty1;
		float_t tx2, ty2;
	} GLFONTCHAR;

	//glFont structure.
	typedef struct
	{
		int32_t Tex;
		int32_t TexWidth, TexHeight;
		int32_t IntStart, IntEnd;
		GLFONTCHAR *Char;
	} GLFONT;

	static GUIManagerC	*CreateInstance();
	static GUIManagerC	*GetInstance() {return sInstance;};
	~GUIManagerC(){};
	//Function prototypes.
	void init();
	void renderGUI();
	void shutdown();
	void updateGUI(double64_t milliseconds);
	//Creates a glFont
	int32_t glFontCreate (GLFONT *Font, char8_t *FileName, int32_t Tex);
	//Deletes a glFont
	void glFontDestroy (GLFONT *Font);
	//Needs to be called before text output
	void glFontBegin (GLFONT *Font);
	void startDefaultFont(int32_t xLocation, int32_t yLocation);
	void endDefaultFont();
	//Needs to be called after text output
	void glFontEnd (void);
	//Draws text with a glFont
	void glFontTextOut (char8_t *String, float_t x, float_t y,	float_t z);
	void setFontColor(float_t red, float_t green, float_t blue);
	//Return the FPS
	float_t returnFPS();
private:
	static GUIManagerC *sInstance;
	GUIManagerC(){};
	//Debug Information.
	void drawDebugInformation();
	//Draw GUI.
	void drawGUI();
	// Frames Per Second.
	int32_t frameCount;
	float_t fps;
	int32_t currentTime;
	int32_t previousTime;
	void calculateFPS();
	//Fonts.
	GLFONT myFont;
	GLFONT numberOfEnemiesFont;
	//Current font.
	GLFONT *glFont;
	//Icon Sprites.
	SpriteC lifeIconSprite;
	SpriteC bombIconSprite;
};
#endif