/*************************************************
**Authors: Kevin Cartrette, Alex Lemke			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary:  Enemy class with modifed render and **
**update to branch from standard enemy behavior.**
*************************************************/

#include "GateEnemy.h"
#include <Windows.h>
#include <stdio.h>
#include <stdlib.h>
#include "stateManager.h"
#include "gl\glu.h"	
#include "ShapeDraw.h"
#include <time.h>
#include "SpriteManager.h"
#include "InputManager.h"
#include "random.h"
#include <math.h>
#include "PlayerManager.h"
#include "player.h"
#include "Bullet.h"
#include "EnemyManager.h"
#include "gamePlayVariables.h"
#include "FieldManager.h"

void GateEnemyC::init(float_t initPosX, float_t initPosY, float_t initVelX, float_t initVelY, int32_t id)
{
	mPosition.x = initPosX;
	mPosition.y = initPosY;
	mVelocity.x = initVelX;
	mVelocity.y = initVelY;

	mRadius = ENEMY_RADIUS;
	scoreValue = GATE_SCORE;
	health = 1;

	enemyColor.x = 0.1f;
	enemyColor.y = 0.1f;
	enemyColor.z = 1.0f;

	//Set a random rotation 
	enemySprite.setRotation(getRangedRandom(0.0f, 360.0f));

	//Point A
	pointAPosition.x = (GATE_LENGTH * cos(enemySprite.returnRotation())) + mPosition.x;
	pointAPosition.y = (GATE_LENGTH * sin(enemySprite.returnRotation())) + mPosition.y;

	//Point B
	pointBPosition.x = (-GATE_LENGTH * cos(enemySprite.returnRotation())) + mPosition.x;
	pointBPosition.y = (-GATE_LENGTH * sin(enemySprite.returnRotation())) + mPosition.y;


#if(ENEMY_TEXTURES)
	enemySprite.setUp(6,0,0,0,64 * 2,64 * 2,0,0,0,64,64);
	enemySprite.setCentered(true);
#endif
}

void GateEnemyC::update(double64_t milliseconds)
{
	if(mEnabled){
		switch(StateManagerC::GetInstance()->getState())
		{
			case StateManagerC::GAMEPLAY:
				FieldManagerC::GetInstance()->applyColor(&mPosition, 10, &enemyColor);
				mVelocity.scaleVector(0.9f);

				pointAPosition.add2DQuickVector(pointAVelocity);
				pointBPosition.add2DQuickVector(pointBVelocity);

				//Setting mPosition to the midpoint
				mPosition.x = (pointAPosition.x + pointBPosition.x)/2.0f;
				mPosition.y = (pointAPosition.y + pointBPosition.y)/2.0f;

				lineRadius = pointAPosition.distance(pointBPosition);
				//Rotate the gate
				//enemySprite.setRotation(enemySprite.returnRotation() + getRangedRandom(-0.01f, 0.01f));

				pointAVelocity.x += (getRangedRandom(-1000.0f,1000.0f));
				pointAVelocity.y += (getRangedRandom(-1000.0f,1000.0f));
				pointBVelocity.x += (getRangedRandom(-1000.0f,1000.0f));
				pointBVelocity.y += (getRangedRandom(-1000.0f,1000.0f));

				pointAVelocity.normalize();
				pointAVelocity.scaleVector(2);
				pointBVelocity.normalize();
				pointBVelocity.scaleVector(2);

#if(ENEMY_TEXTURES)
				enemySprite.setPosition(mPosition.x, mPosition.y, 0);
#endif
				DoCollisions();
				break;
		}
	}
}

void GateEnemyC::render()
{
	if(mEnabled){
		//Push Matrix
		glPushMatrix();
#if(ENEMY_TEXTURES)
		//SpriteManagerC::GetInstance()->drawSprite(enemySprite);

		DrawCircle(3, pointAPosition.x,pointAPosition.y, 1.0f ,0.6f,0,false);
		DrawCircle(3, pointBPosition.x,pointBPosition.y, 1.0f ,0.6f,0,false);

		DrawLine(pointAPosition.x, pointAPosition.y, pointBPosition.x, pointBPosition.y, 1.0f, 1.0f, 1.0f);
#else
		char8_t red = (char8_t)((mEnemyColor>>16)& 0xFF);
		char8_t green = (char8_t)((mEnemyColor>>8) & 0xFF);
		char8_t blue = (char8_t)((mEnemyColor>>0) & 0xFF);
		bool filledVal;

		filledVal = true;

		DrawCircle(mRadius, mPosition.x,mPosition.y,red,green,blue,filledVal);
#endif
		//Pop Matrix
		glPopMatrix();
	}
}

bool GateEnemyC::checkIfBulletsCollide(int32_t playerNumber)
{
	PlayerC *temporaryPlayerPtr = PlayerManagerC::GetInstance()->getPlayerPtr(playerNumber);
	if(temporaryPlayerPtr->getEnabled()) // If the player is enabled
	{
		for(int b = 0; b < temporaryPlayerPtr->getNumBullets(); b++)
		{
			BulletC *temporaryBulletPtr = temporaryPlayerPtr->getBulletPtr(b);
			if(temporaryBulletPtr->getEnabled()) // If the bullet is enabled
			{
				//if(mPosition.distance( mPosition, (*(temporaryBulletPtr->getPosition()))) < (mRadius + temporaryBulletPtr->getRadius()))
				//{
				Vector3C bulletPosition = (*(temporaryBulletPtr->getPosition()));
				//if(checkIfCollideRadial((mRadius + temporaryBulletPtr->getRadius()), bulletPosition))
				
				if(checkIfCollideRadial(temporaryBulletPtr->getRadius(), bulletPosition, 1, pointAPosition))
				{
					//Bullet has collided so destroy it
					temporaryBulletPtr->disable();
					return(true);
				}
				else if(checkIfCollideRadial(temporaryBulletPtr->getRadius(), bulletPosition, 1, pointBPosition))
				{
					//Bullet has collided so destroy it
					temporaryBulletPtr->disable();
					return(true);
				}
				else if(checkIfCollideRadial(temporaryBulletPtr->getRadius(), bulletPosition, lineRadius/2.0f, mPosition)) //Bullets bounce off!
				{
					Vector3C bulletDirection = *temporaryBulletPtr->getVelocity();
					float_t bulletSpeed = bulletDirection.getLength();

					//Normalize the bullet direction
					bulletDirection.normalize();

					//Have to mirror the velocity accross the flipped normal to the line
					Vector3C gateNormal;
					gateNormal.y = pointBPosition.x - pointAPosition.x;
					gateNormal.x = pointBPosition.y - pointAPosition.y;
					gateNormal.normalize();

					//Flip the bullets velocity
					//bulletDirection.x = -bulletDirection.x;
					//bulletDirection.y = -bulletDirection.y;

					//Get the dot product between the gate wall and the bullet's direction;
					float_t angle = acosf(gateNormal.dotProduct(bulletDirection));
					//angle *= 2.0f;
					angle = 360.0f - angle;
					bulletDirection.x = (float_t)((bulletDirection.x * cosf(angle)) + (bulletDirection.y * -sinf(angle)));
					bulletDirection.y = (float_t)((bulletDirection.x * sinf(angle)) + (bulletDirection.y * cosf(angle)));
					bulletDirection.normalize();

					bulletDirection.scaleVector(bulletSpeed);

					temporaryBulletPtr->setVelocity(bulletDirection);
				}
			}
		}
	}
	return(false);
}

bool GateEnemyC::checkIfPlayerPositionCollide(double radius, int32_t playerNumber)
{
	if(this->mEnabled){
		Vector3C position = *PlayerManagerC::GetInstance()->getPlayerPtr(playerNumber)->getPosition();
		if(checkIfCollideRadial(radius, position, 1, pointAPosition))
		{
			return(true);
		}
		else if(checkIfCollideRadial(radius, position, 1, pointBPosition))
		{
			return(true);
		}
		else if(checkIfCollideRadial(radius, position, lineRadius/2.0f, mPosition)) //Gate Explodes
		{
			this->disable();
			//EnemyManagerC::GetInstance()->checkEnemyRadialCollision(lineRadius, mPosition, true);
			//return(true);
			EnemyC* temporaryEnemy;
			for(int32_t i = 0; i <EnemyManagerC::GetInstance()->getNumEnemies(); i++)
			{
				temporaryEnemy = EnemyManagerC::GetInstance()->getEnemyPtr(i);
				Vector3C temporaryEnemyPositon = *(temporaryEnemy->getPosition());
				
				//Distance to gate enemy
				float_t distanceToGateCenter = mPosition.distance(temporaryEnemyPositon);
				float_t distanceToPointA = mPosition.distance(pointAPosition);
				float_t distanceToPointB = mPosition.distance(pointBPosition);
				//Exploding enemies around the gate!
				if(distanceToGateCenter < (lineRadius * 2.0f)) //Center
				{
					EnemyManagerC::GetInstance()->getEnemyPtr(i)->disable();
					EnemyManagerC::GetInstance()->enemyDeathEvent(playerNumber, temporaryEnemyPositon, temporaryEnemy->getScoreValue());
				}
				else if(distanceToPointA  < (lineRadius * 0.25f)) //End Point A
				{
					EnemyManagerC::GetInstance()->getEnemyPtr(i)->disable();
					EnemyManagerC::GetInstance()->enemyDeathEvent(playerNumber, temporaryEnemyPositon, temporaryEnemy->getScoreValue());
				}
				else if(distanceToPointB  < (lineRadius * 0.25f)) //End Point B
				{
					EnemyManagerC::GetInstance()->getEnemyPtr(i)->disable();
					EnemyManagerC::GetInstance()->enemyDeathEvent(playerNumber, temporaryEnemyPositon, temporaryEnemy->getScoreValue());
				}
			}
		}
	}

	return(false);
}