/*************************************************
**Authors: Alex Lemke, Kevin Cartrette			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary: A point on the grid.                 **
*************************************************/

#include "GridPoint.h"
#include "field.h"
#include "FieldManager.h"
#include "gamePlayVariables.h"

 GridPointC::GridPointC()
 {
	 locked = false;
	 currentColor.x = colorChange.x = 0.0f;
	 currentColor.y = colorChange.y = 0.0f;
	 currentColor.z = colorChange.z = 0.0f;
 }

void GridPointC::setPosition(Vector3C newPosition)
{
	currentPosition.x = newPosition.x;
	currentPosition.y = newPosition.y;

	initalPosition.x = newPosition.x;
	initalPosition.y = newPosition.y;
}

void GridPointC::applyForce(Vector3C newForce)
{
	velocity.add2DQuickVector(newForce);
}

void GridPointC::update(DWORD milliseconds)
{
	//Find the distance from the inital starting point.
	if(locked == false){
		if(velocity.getLength() != 0){
			currentPosition.add2DQuickVector(velocity);

			CollideField();

			//Needs some friction
			velocity.scaleVector(0.992f);
		}

		//Here is a small push towards correcting yourself.
		Vector3C smallPush;
		smallPush.x = currentPosition.x;
		smallPush.y = currentPosition.y;
		smallPush.subtract2DQuickVector(initalPosition);
		smallPush.scaleVector(-RETURN_PUSH_AMOUNT);

		applyForce(smallPush);
	}
	else{
		velocity.scaleVector(0.99f);
	}
}

Vector3C GridPointC::getCurrentPosition()
{
	return(currentPosition);
}

Vector3C  GridPointC::getCurrentVelocity()
{
	return velocity;
}

Vector3C GridPointC::getInitalPosition()
{
	return(initalPosition);
}

void GridPointC::setLocked(bool newLocked)
{
	locked = newLocked;
}

void GridPointC::CollideField(void)
{
	FieldC *field = FieldManagerC::GetInstance()->getFieldPtr();

	float_t rightSide = +((float_t)field->getWidth()/TWO);
	float_t leftSide = -((float_t)field->getWidth()/TWO);
	float_t topSide = +((float_t)field->getHeight()/TWO);
	float_t bottomSide = -((float_t)field->getHeight()/TWO);
	
	//Box collision
	if(currentPosition.x <= leftSide  )
	{
		velocity.x = -velocity.x;
		currentPosition.x = leftSide;
	}
	if(currentPosition.x >= rightSide  )
	{
		velocity.x = -velocity.x;
		currentPosition.x = rightSide;
	}
	if(currentPosition.y <= bottomSide  )
	{
		velocity.y = -velocity.y;
		currentPosition.y = bottomSide;
	}
	if(currentPosition.y >= topSide  )
	{
		velocity.y = -velocity.y;
		currentPosition.y = topSide;
	}
}

void GridPointC::clearVelocity()
{
	velocity.x = 0;
	velocity.y = 0;
}
Vector3C* GridPointC::getCurrentColor()
{
	return(&currentColor);
}

Vector3C* GridPointC::getColorChange()
{
	return(&colorChange);
}

void GridPointC::setCurrentColor(Vector3C* newColor)
{
	currentColor.x = newColor->x;
	currentColor.y = newColor->y;
	currentColor.z = newColor->z;
}

void GridPointC::decreaseColor(float_t scaleAmount)
{
	if(colorChange.x > -0.1f)
		colorChange.x -= 0.0005f;
	if(colorChange.y > -0.1f)
		colorChange.y -= 0.0005f;
	if(colorChange.z > -0.1f)
		colorChange.z -= 0.0005f;

	if(currentColor.x < 0.0f)
		currentColor.x = 0.0f;
	if(currentColor.y < 0.0f)
		currentColor.y = 0.0f;
	if(currentColor.z < 0.0f)
		currentColor.z = 0.0f;

}

Vector3C* GridPointC::getVelocity()
{
	return(&velocity);
}

void GridPointC::mergeColorChange(Vector3C* newColor)
{
	colorChange.x = (colorChange.x + newColor->x);
	colorChange.y = (colorChange.y + newColor->y);
	colorChange.z = (colorChange.z + newColor->z);
}

void GridPointC::appyPushColorChange(Vector3C* newColor)
{
	colorChange.x = (colorChange.x + newColor->x);
	colorChange.y = (colorChange.y + newColor->y);
	colorChange.z = (colorChange.z + newColor->z);

	currentColor.x = newColor->x;
	currentColor.y = newColor->y;
	currentColor.z = newColor->z;
}

void GridPointC::updateColor()
{
	decreaseColor(0.95f);

	currentColor.x = (colorChange.x + currentColor.x)*GRID_COLOR_BLEND_AMOUNT;
	currentColor.y = (colorChange.y + currentColor.y)*GRID_COLOR_BLEND_AMOUNT;
	currentColor.z = (colorChange.z + currentColor.z)*GRID_COLOR_BLEND_AMOUNT;

	colorChange.x = currentColor.x;
	colorChange.y = currentColor.y;
	colorChange.z = currentColor.z;
}

void GridPointC::setColorChange(Vector3C* newColorChange)
{
	colorChange.x = newColorChange->x;
	colorChange.y = newColorChange->y;
	colorChange.z = newColorChange->z;
}

void GridPointC::setColor(Vector3C* newColor)
{
	currentColor.x = newColor->x;
	currentColor.y = newColor->y;
	currentColor.z = newColor->z;
	colorChange.x = 0.0f;
	colorChange.y = 0.0f;
	colorChange.z = 0.0f;
}

void GridPointC::setNumberOfColorSums(float_t newSum)
{
	numberOfColorSumValues = newSum;
}

float_t GridPointC::returnNumberOfColorSums()
{
	return(numberOfColorSumValues);
}