/*************************************************
**Authors: Alex Lemke, Kevin Cartrette			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary: The input manager. Handles keyboard  **
**			and mouse input.					**
*************************************************/

#include <windows.h>									
#include "InputManager.h"
#include "Vector2.h"
#include "SpriteManager.h"

InputManagerC* InputManagerC::sInstance = NULL;

InputManagerC *InputManagerC::CreateInstance()
{
	if(sInstance!=NULL)return sInstance;
	else
	sInstance = new InputManagerC();
	return sInstance;
}

void InputManagerC::update()
{
	//Setting Previous keys to current keys
	for(int32_t i = 0; i < 256; i++)
	{
		previousKeys[i] = currentKeys[i];
	}
	Vector2C currentMousePosition = returnMouseGamePosition();
	mouseSprite.setPosition(currentMousePosition.x,currentMousePosition.y,0);
}

void InputManagerC::init()
{
	mouseSprite.setUp(2,0,0,0,32 * 2,32* 2,0,0,0,32,32);
	mouseSprite.setCentered(true);
}

void InputManagerC::draw()
{
	SpriteManagerC::GetInstance()->drawSprite(mouseSprite);
}

void InputManagerC::setKeys(BOOL newKeys[256])
{
	for(int32_t i = 0; i < 256; i++)
	{
		currentKeys[i] = newKeys[i];
	}
}

void InputManagerC::resetKeys()
{
	for(int32_t i = 0; i < 256; i++)
	{
		currentKeys[i] = false;
	}
}

void InputManagerC::setMouse(bool8_t newLClick, bool8_t newRClick, int32_t newMouseX, int32_t newMouseY)
{
	//Setting Click Values.
	mouseLClick = newLClick;
	mouseRClick = newRClick;

	mouseX = newMouseX;
	mouseY = newMouseY;
}

bool8_t InputManagerC::isLeftMouseButtonDown()
{
	return(mouseLClick);
}

Vector2C InputManagerC::returnMouseScreenPosition()
{
	Vector2C mousePosition;
	mousePosition.x = (float_t)mouseX;
	mousePosition.y = (float_t)mouseY;
	return(mousePosition);
}

Vector2C InputManagerC::returnMouseGamePosition()
{
	Vector2C mousePosition;
	mousePosition.x = (float_t)mouseX;
	mousePosition.y = (float_t)mouseY;

	mousePosition.x /= windowWidth;
	mousePosition.y /= windowHeight;

	mousePosition.x *= gameWorldWidth;
	mousePosition.y *= gameWorldHeight;

	mousePosition.x -= (gameWorldWidth/2);
	mousePosition.y -= (gameWorldHeight/2);

	mousePosition.y = -mousePosition.y;

	return(mousePosition);
}

void InputManagerC::setScreenInformation(float_t newWindowWidth, float_t newWindowHeight, float_t newGameWorldWidth, float_t newGameWorldHeight)
{
	windowWidth = newWindowWidth;
	windowHeight = newWindowHeight;

	gameWorldWidth = newGameWorldWidth;
	gameWorldHeight = newGameWorldHeight;
}