#include "MenuManager.h"
#include "GUIManager.h"
#include "stateManager.h"
#include "InputManager.h"
#include "openglframework.h"
#include "Vector2.h"
#include "gamePlayVariables.h"
#include "PlayerManager.h"
#include "SpriteManager.h"
#include "FieldManager.h"
#include "PlayerManager.h"
#include "player.h"

MenuManagerC* MenuManagerC::mInstance = 0;

MenuManagerC::MenuManagerC()
{
}

MenuManagerC::~MenuManagerC()
{
}

MenuManagerC *MenuManagerC::CreateInstance()
{
	if(mInstance!=0)return mInstance;
	else
	mInstance = new MenuManagerC();
	return mInstance;
}

void MenuManagerC::init()
{
	mainMenuItems();
	currentTime = 0;
	timeBetweenMenuUpdates = MENU_TIME_BETWEEN_UPDATES;

	titleSprite.setUp(11, -500, 500, 0, 1000, 100, 0, 0, 0, 712, 64);
}

void MenuManagerC::mainMenuItems()
{
	shutdown();
	Vector3C position;
	currentActiveItem = 0;
	currentActiveMenu = MAIN;
	position.x = MENU_START_X;
	position.y = MENU_START_Y;
	newMenuItem("Survival Mode", position, 1.0f, 0.5f, 0.1f, true);
	newMenuItem("Wave Mode", position, 1.0f, 0.5f, 0.1f, false);
	newMenuItem("Worm Hole Mode", position, 1.0f, 0.5f, 0.1f, false);
	newMenuItem("Pacifism Mode", position, 1.0f, 0.5f, 0.1f, false);
	newMenuItem("Exit Game", position, 1.0f, 0.5f, 0.1f, false);
}

void MenuManagerC::scoreMenuItems()
{
	shutdown();
	Vector3C position;
	currentActiveItem = 0;
	currentActiveMenu = SCORE;
	position.x = MENU_START_X;
	position.y = MENU_START_Y;

	//Output the score
	sprintf_s(scoreString,"SCORE:: %d \n", PlayerManagerC::GetInstance()->getPlayerPtr(0)->getScore());

	newMenuItem(scoreString, position, 1.0f, 0.0f, 0.0f, true);
}

int32_t MenuManagerC::newMenuItem(char8_t* name, Vector3C initPosition, float_t red, float_t blue, float_t green, bool active)
{
	int32_t menuItemSpot;
	menuItemVector.push_back(new MenuItemC);
	menuItemSpot = menuItemVector.size() - 1;
	initPosition.y += MENU_ITEM_OFFSET * menuItemSpot;
	menuItemVector[menuItemSpot]->init(name,initPosition,red,blue,green,false);
	return menuItemSpot;
}

void MenuManagerC::update(double64_t milliseconds)
{
	switch(StateManagerC::GetInstance()->getState())
	{
		case StateManagerC::MAIN_MENU:
			if(currentActiveMenu != MAIN)
			{
				mainMenuItems();
			}
			currentTime += milliseconds;
			if(currentTime >= timeBetweenMenuUpdates)
			{
				if(InputManagerC::GetInstance()->currentKeys['W'] == (int32_t)true)
				{
					currentActiveItem--;
					if(currentActiveItem < 0)
					{
						currentActiveItem = (int32_t)menuItemVector.size()-1;
					}
					currentTime = 0;
				}
				else if(InputManagerC::GetInstance()->currentKeys['S'] == (int32_t)true)
				{
					currentActiveItem++;
					if(currentActiveItem > (int32_t)menuItemVector.size()-1)
					{
						currentActiveItem = 0;
					}
					currentTime = 0;
				}
				else if(InputManagerC::GetInstance()->currentKeys[' '] == (int32_t)true)
				{
					PlayerManagerC::GetInstance()->getPlayerPtr(0)->reset();
					//playerPtrs[i]->reset();
					FieldManagerC::GetInstance()->resetGrid();
					if(currentActiveItem == SURVIVAL_ITEM - 1)
					{
						StateManagerC::GetInstance()->setState(StateManagerC::GAMEPLAY);
						StateManagerC::GetInstance()->setGameMode(StateManagerC::SURVIVAL_MODE);
					}
					else if(currentActiveItem == WAVE_ITEM - 1)
					{
						StateManagerC::GetInstance()->setState(StateManagerC::GAMEPLAY);
						StateManagerC::GetInstance()->setGameMode(StateManagerC::WAVE_MODE);
					}
					else if(currentActiveItem == WORM_HOLE_ITEM - 1)
					{
						StateManagerC::GetInstance()->setState(StateManagerC::GAMEPLAY);
						StateManagerC::GetInstance()->setGameMode(StateManagerC::WORM_HOLE_MODE);
					}
					else if(currentActiveItem == PACIFISM_ITEM - 1)
					{
						StateManagerC::GetInstance()->setState(StateManagerC::GAMEPLAY);
						StateManagerC::GetInstance()->setGameMode(StateManagerC::PACIFISM_MODE);
					}
					else if(currentActiveItem == EXIT_ITEM - 1)
					{
						TerminateApplication (g_window);
					}
				}

				for(int32_t i = 0; i < (int32_t)menuItemVector.size(); i++)
				{
					if(currentActiveItem == i)
					{
						menuItemVector[i]->setActive(true);
					}
					else
					{
						menuItemVector[i]->setActive(false);
					}
					menuItemVector[i]->update(milliseconds);
				}
			}
			break;
		case StateManagerC::SCORE_MENU:
			if(currentActiveMenu != SCORE)
			{
				scoreMenuItems();
			}
			currentTime += milliseconds;
			if(currentTime >= timeBetweenMenuUpdates)
			{
				if(InputManagerC::GetInstance()->currentKeys[' '] == (int32_t)true)
				{
					StateManagerC::GetInstance()->setState(StateManagerC::MAIN_MENU);
				}
				currentTime = 0;
			}

			break;
	}
}

void MenuManagerC::render()
{
	switch(StateManagerC::GetInstance()->getState())
	{
		case StateManagerC::MAIN_MENU:
		case StateManagerC::SCORE_MENU:
			for (menuItemIterator = menuItemVector.begin(); menuItemIterator != menuItemVector.end(); menuItemIterator++)
			{
				(*menuItemIterator)->render();
			}

			glLoadIdentity();
			
			glPushMatrix();
			//Render Title Sprite
			SpriteManagerC::GetInstance()->drawSprite(titleSprite);
			glPopMatrix();

			break;
	}
}

void MenuManagerC::shutdown()
{
	for (menuItemIterator = menuItemVector.begin(); menuItemIterator != menuItemVector.end();)
	{
		delete(*menuItemIterator);
		menuItemIterator = menuItemVector.erase(menuItemIterator);
	}
}
