/*************************************************
**Authors: Kevin Cartrette, Alex Lemke			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary: Menu manager which handles displaying**
**all menu items.								**
*************************************************/

#ifndef MENUMANAGER_H
#define MENUMANAGER_H

#include "MenuItem.h"
#include "baseTypes.h"
#include <vector>
#include "Sprite.h"

using namespace std;

/*************************************************
**Summary: Menu item class which stores object  **
**info about what to draw on the screen.		**
**Authors: Kevin Cartrette, Alex Lemke			**
*************************************************/
class MenuManagerC
{
public:
	//Menu enum's for valid menu types.
	enum Menu
	{
		INVALID_MENU,
		MAIN,
		SCORE,
		MAX_MENU
	};
	//MenuItem enum's for valid menu item types.
	enum MenuItem
	{
		INVALID_ITEM,
		SURVIVAL_ITEM,
		WAVE_ITEM,
		WORM_HOLE_ITEM,
		PACIFISM_ITEM,
		EXIT_ITEM,
		MAX_ITEM
	};

	static MenuManagerC *CreateInstance();
	static MenuManagerC *GetInstance() {return mInstance;};

	MenuManagerC();
	~MenuManagerC();
	//Function prototypes.
	void init();
	void update(double64_t milliseconds);
	void render();
	void shutdown();
	void mainMenuItems();
	void scoreMenuItems();
	int32_t newMenuItem(char8_t* name, Vector3C initPosition, float_t red, float_t blue, float_t green, bool active);
private:
	char8_t scoreString[100];

	static MenuManagerC *mInstance;
	//Vector of menu item class objects.
	vector<MenuItemC*> menuItemVector;
	vector<MenuItemC*>::iterator menuItemIterator;
	//Time variables to handle update time values.
	double64_t currentTime;
	double64_t timeBetweenMenuUpdates;
	int32_t currentActiveMenu;
	int32_t currentActiveItem;
	//Title Sprite.
	SpriteC titleSprite;
};
#endif