/*************************************************
**Authors: Kevin Cartrette, Alex Lemke			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary:  Basic particle system manager deals **
**with the processing of all particles in game. **
*************************************************/

#ifndef PARTICLE_H
#define PARTICLE_H
#include "baseTypes.h"
#include "Vector3.h"
#include "Sprite.h"

/*************************************************
**Summary: Particle class which is the object	**
**the particle system is rendering.				**
**Authors: Kevin Cartrette, Alex Lemke			**
*************************************************/
class ParticleC
{
public:
	ParticleC();
	~ParticleC();
	//Function prototypes.
	void init(uint32_t initTextureId, Vector3C initPosition, Vector3C initVelocity, Vector3C initAngularVelocity, Vector3C initColor, float_t initSize, float_t initTimeToLive);
	void draw();
	bool update(double64_t milliseconds);
	void shutDown();
	bool isAlive();
	void setRandomColor(bool newRandomColor);
	bool isRandomColor();
	void setBounceOffWalls(bool newBounceOffWalls);
	bool getBounceOffWalls();
	void setAlive(bool newAlive);
	void collideField();
private:
	//Attributes of particle.
	Vector3C position;
	Vector3C velocity;
	Vector3C angularVelocity;
	Vector3C color;
	float_t size;
	float_t timeToLive;
	double64_t currentTime;
	bool alive;
	bool randomColor;
	bool bounceOffWalls;
	uint32_t textureId;
	//Particles texture if in use.
	SpriteC particleSprite;
};

#endif