/*************************************************
**Authors: Kevin Cartrette, Alex Lemke			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary:  Basic particle system manager deals **
**with the processing of all particles in game. **
*************************************************/

#include <vector>
#include "ParticleEmitter.h"
#include "ParticleCircleEmitter.h"
#include "random.h"
#include "gamePlayVariables.h"

ParticleCircleEmitterC::ParticleCircleEmitterC()
{
}

ParticleCircleEmitterC::~ParticleCircleEmitterC()
{
}

void ParticleCircleEmitterC::update(double64_t milliseconds)
{
	currentTime += milliseconds;

	//printf("current time %lf\n", currentTime);
	if(currentTime >= timeBetweenUpdates)
	{
		bool isAlive;
		for (particleIterator = particleVector.begin(); particleIterator != particleVector.end();)
		{
			isAlive = (*particleIterator)->update(milliseconds);
			if(isAlive != true)
			{
				(*particleIterator)->shutDown();
				particleIterator = particleVector.erase(particleIterator);
			}
			else
			{
				if((*particleIterator)->update(milliseconds) == false){
					delete(*particleIterator);
					particleIterator = particleVector.erase(particleIterator);
				}
				else
					particleIterator++;
			}
		}

		if(emit == true && active == true && (((int32_t)particleVector.size()) < minParticles))
		{
			int32_t difference = minParticles - ((int32_t)particleVector.size());

			for(int32_t i = 0; i < difference; i++)
			{
				particleVector.push_back(new ParticleC);
				int32_t particleSpot = particleVector.size() - 1;
				
				
				//Angular Velocity
				Vector3C newAngularVelocity;
				newAngularVelocity.x = getRangedRandom(angularVelocity.x,TWO *PI_DEFINE);
				newAngularVelocity.y = getRangedRandom(angularVelocity.y,TWO *PI_DEFINE);

				//Translation Velocity
				Vector3C newTranslationVelocity;
				
				newTranslationVelocity.x = cos(newAngularVelocity.x);// * getRangedRandom(velocity.x, velocity.x + getRangedRandom(0,10));
				newTranslationVelocity.y = sin(newAngularVelocity.x);// * getRangedRandom(velocity.y, velocity.y + getRangedRandom(0,10));
				newTranslationVelocity.scaleVector(velocity.getLength() + getRangedRandom(0,10));

				//newTranslationVelocity.x = cos(newAngularVelocity.x) * getRangedRandom(velocity.x, velocity.x + getRangedRandom(0,10));
				//newTranslationVelocity.y = sin(newAngularVelocity.y) * getRangedRandom(velocity.y, velocity.y + getRangedRandom(0,10));
				
				particleVector[particleSpot]->init(PARTICLE_CIRCLE_EMITTER_TEXTURE_ID,position,newTranslationVelocity,newAngularVelocity,color,size,
					getRangedRandom(PARTICLE_CIRCLE_EMITTER_MIN_TIME_TO_LIVE,PARTICLE_CIRCLE_EMITTER_MAX_TIME_TO_LIVE));
				
				if(particlesBounceOffWalls == true)
				{
					particleVector[particleSpot]->setBounceOffWalls(true);
				}
				numberParticles++;
			}
		}

		if(emit == true && active == true && (((int32_t)particleVector.size()) < maxParticles))
		{
			particleVector.push_back(new ParticleC);
			int32_t particleSpot = particleVector.size() - 1;
			Vector3C newAngularVelocity;
			Vector3C newVelocity;
			newAngularVelocity.x = getRangedRandom(angularVelocity.x,TWO *PI_DEFINE);
			newAngularVelocity.y = getRangedRandom(angularVelocity.y,TWO *PI_DEFINE);
			newVelocity.x = cos(newAngularVelocity.x) * getRangedRandom(velocity.x, velocity.x + getRangedRandom(0,10));
			newVelocity.y = sin(newAngularVelocity.y) * getRangedRandom(velocity.y, velocity.y + getRangedRandom(0,10));
			particleVector[particleSpot]->init(PARTICLE_CIRCLE_EMITTER_TEXTURE_ID,position,newVelocity,newAngularVelocity,color,size,
				getRangedRandom(PARTICLE_CIRCLE_EMITTER_MIN_TIME_TO_LIVE,PARTICLE_CIRCLE_EMITTER_MAX_TIME_TO_LIVE));
			if(particlesBounceOffWalls == true)
			{
				particleVector[particleSpot]->setBounceOffWalls(true);
			}
			numberParticles++;
		}


		//If time based, set active status to false.
		if(timeBased == true)
		{
			totalTimeAlive += currentTime;
			if(totalTimeAlive >= totalTimeToEmit && emit == true)
			{
				emit = false; //stop emitting particles
			}
		}

		if(emit == false && (((int)particleVector.size()) <= 0))
		{
			active = false;
		}

		currentTime = 0;
	}
}