/*************************************************
**Authors: Kevin Cartrette, Alex Lemke			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary:  Basic particle system manager deals **
**with the processing of all particles in game. **
*************************************************/

#include "ParticleEmitter.h"
#include "random.h"

ParticleEmitterC::ParticleEmitterC()
{
}

ParticleEmitterC::~ParticleEmitterC()
{
}

void ParticleEmitterC::init(double64_t initTimeBetweenUpdates, double64_t initTotalTimeToEmit, Vector3C initPosition, Vector3C initVelocity, Vector3C initAngularVelocity,
	float_t initSize, int32_t initMinParticles, int32_t initMaxParticles)
{
	position.setEqualToVector(initPosition);
	velocity.setEqualToVector(initVelocity);
	angularVelocity.setEqualToVector(initAngularVelocity);
	minParticles = initMinParticles;
	maxParticles = initMaxParticles;
	active = true;
	timeBased = true;
	emit = true;
	particlesBounceOffWalls = false;
	timeBetweenUpdates = initTimeBetweenUpdates;
	totalTimeToEmit = initTotalTimeToEmit;
	currentTime = 0;
	numberParticles = 0;
	totalTimeAlive = 0;
	size = initSize;

	color.x = getRangedRandom(0.0f,1.0f);
	color.y = getRangedRandom(0.0f,1.0f);
	color.z = getRangedRandom(0.0f,1.0f);
}

void ParticleEmitterC:: renderParticles()
{
	for (particleIterator = particleVector.begin(); particleIterator != particleVector.end(); particleIterator++)
	{
		(*particleIterator)->draw();
	}
}

void ParticleEmitterC::setPosition(Vector3C newPosition)
{
	position.setEqualToVector(newPosition);
}

void ParticleEmitterC::setVelocity(Vector3C newVelocity)
{
	velocity.setEqualToVector(newVelocity);
}

void ParticleEmitterC::setAngularVelocity(Vector3C newAngularVelocity)
{
	angularVelocity.setEqualToVector(newAngularVelocity);
}

void ParticleEmitterC::setActiveStatus(bool newActive)
{
	active = newActive;
}

bool ParticleEmitterC::isActive()
{
	return active;
}

void ParticleEmitterC::setTimeBased(bool newTimeBased)
{
	timeBased = newTimeBased;
}

void ParticleEmitterC::setParticleBounceOffWalls(bool newParticlesBounceOffWalls)
{
	particlesBounceOffWalls = newParticlesBounceOffWalls;
}

bool ParticleEmitterC::isTimeBased()
{
	return timeBased;
}

void ParticleEmitterC::setEmit(bool newEmit)
{
	emit = newEmit;
}
bool ParticleEmitterC::isEmitting()
{
	return emit;
}

int32_t  ParticleEmitterC::getNumberParticles()
{
	numberParticles = particleVector.size()-1;
	return numberParticles;
}

void ParticleEmitterC::shutdown()
{
	for (particleIterator = particleVector.begin(); particleIterator != particleVector.end();)
	{
		delete(*particleIterator);
		particleIterator = particleVector.erase(particleIterator);
	}
}

void ParticleEmitterC::setColor(Vector3C* newColor)
{
	color.x = newColor->x;
	color.y = newColor->y;
	color.z = newColor->z;
}