/*************************************************
**Authors: Kevin Cartrette, Alex Lemke			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary:  Basic particle system manager deals **
**with the processing of all particles in game. **
*************************************************/

#ifndef PARTICLEEMITTER_H
#define PARTICLEEMITTER_H
#include "Particle.h"
#include "baseTypes.h"
#include "Vector3.h"
#include <vector>
#include "baseTypes.h"

using namespace std;

/*************************************************
**Summary: Particle Emitter is a base class for **
**the basis of all emitters.					**
**Authors: Kevin Cartrette, Alex Lemke			**
*************************************************/
class ParticleEmitterC
{
public:
	ParticleEmitterC();
	~ParticleEmitterC();
	//Function prototypes.
	void init(double64_t initTimeBetweenUpdates, double64_t initTotalTimeToEmit, Vector3C initPosition, Vector3C initVelocity, 
		Vector3C initAngularVelocity, float_t initSize, int32_t initMinParticles, int32_t initMaxParticles);
	virtual void update(double64_t milliseconds){};
	void renderParticles();
	void setPosition(Vector3C newPosition);
	void setVelocity(Vector3C newVelocity);
	void setAngularVelocity(Vector3C newAngularVelocity);
	void setActiveStatus(bool newActive);
	bool isActive();
	void setTimeBased(bool newTimeBased);
	bool isTimeBased();
	void setEmit(bool newEmit);
	bool isEmitting();
	void setParticleBounceOffWalls(bool newParticlesBounceOffWalls);
	void shutdown();
	int32_t getNumberParticles();
	void setColor(Vector3C* newColor);
protected:
	//Attributes of all particle emitters.
	Vector3C position;
	Vector3C velocity;
	Vector3C angularVelocity;
	Vector3C color;
	//Handle game update timing events.
	double64_t currentTime;
	double64_t timeBetweenUpdates;
	double64_t totalTimeToEmit;
	double64_t totalTimeAlive;
	//Size of particle to be emitted.
	float_t size;
	//Vector of particle class objects.
	vector<ParticleC*> particleVector;
	vector<ParticleC*>::iterator particleIterator;
	//Attributes and limitations on emitting of particles.
	int32_t numberParticles;
	int32_t minParticles;
	int32_t maxParticles;
	bool active;
	bool timeBased;
	bool emit;
	bool particlesBounceOffWalls;
};
#endif