/*************************************************
**Authors: Kevin Cartrette, Alex Lemke			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary:  Basic particle system manager deals **
**with the processing of all particles in game. **
*************************************************/

#ifndef H_PARTICLEMANAGER
#define H_PARTICLEMANAGER

#include "baseTypes.h"
#include "ParticleEmitter.h"
#include "ParticlePointEmitter.h"
#include "ParticleCircleEmitter.h"
#include <vector>

using namespace std;

/*************************************************
**Summary: Particle Manager is the class which  **
**handles all particle emitters.				**
**Authors: Kevin Cartrette, Alex Lemke			**
*************************************************/
class ParticleManagerC
{
public:
	//Enum of all valid types of particle emitters.
	enum Emitters
	{
		Invalid_Emitter,
		PointEmitter,
		CircleEmitter,
		Max_Emitter
	};
	//Function prototypes.
	static ParticleManagerC *CreateInstance();
	static ParticleManagerC *GetInstance() {return pInstance;};
	void init();
	void updateParticles(double64_t milliseconds);
	void renderParticles();
	int32_t newEmitter(Emitters emitter, double64_t initTimeBetweenUpdates, double64_t initTotalTimeToEmit, Vector3C initPosition, Vector3C initVelocity, Vector3C initAngularVelocity,
		float_t initSize, int32_t initMinParticles, int32_t initMaxParticles, Vector3C* particleColor);
	void shutdown();
	void setTimeBased(int32_t indexEmitter, bool newTimeBased);
	void setParticleBounceOffWalls(int indexEmitter, bool newParticleBounceOffWalls);
	void updateParticleEmitters(double64_t currentTime);

	int32_t getTotalNumberOfParticles();


private:
	static ParticleManagerC *pInstance;
	//Update loop timer variables.
	double64_t currentTime;
	double64_t timeBetweenUpdates;
	double64_t timeSinceLastMenuUpdate;
	double64_t timeBetweenMenuUpdates;
	//Current number of particles
	int32_t currentParticleCount;
	//Vector of particle emitter objects.
	vector<ParticleEmitterC*> particleEmitterVector;
	vector<ParticleEmitterC*>::iterator particleEmitterIterator;


	//Temporary Vector 3 used for almost anything and everything
	Vector3C temp;
};
#endif