/*************************************************
**Authors: Kevin Cartrette, Alex Lemke			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary:  Basic particle system manager deals **
**with the processing of all particles in game. **
*************************************************/

#include <vector>
#include "ParticleEmitter.h"
#include "ParticlePointEmitter.h"
#include "random.h"
#include "gamePlayVariables.h"

ParticlePointEmitterC::ParticlePointEmitterC()
{
}

ParticlePointEmitterC::~ParticlePointEmitterC()
{
}

void ParticlePointEmitterC::update(double milliseconds)
{
	currentTime += milliseconds; //Update the current time till last update
	if(currentTime >= timeBetweenUpdates && active == true)
	{
		//If time based, set active status to false.
		if(timeBased == true)
		{
			totalTimeAlive += currentTime;
			if(totalTimeAlive >= totalTimeToEmit)
			{
				emit = false;
			}
		}

		bool isAlive;
		for (particleIterator = particleVector.begin(); particleIterator != particleVector.end();)
		{
			isAlive = (*particleIterator)->update(milliseconds);
			if(isAlive != true)
			{
				(*particleIterator)->shutDown();
				particleIterator = particleVector.erase(particleIterator);
			}
			else
			{
				(*particleIterator)->update(milliseconds);
				particleIterator++;
			}
		}

		if(emit == true && active == true && (((int32_t)particleVector.size()) < minParticles))
		{
			int32_t difference = minParticles - ((int32_t)particleVector.size());

			for(int32_t i = 0; i < difference; i++)
			{
				particleVector.push_back(new ParticleC);
				int32_t particleSpot = particleVector.size() - 1;
				Vector3C newAngularVelocity;
				Vector3C newVelocity;
				newAngularVelocity.x = getRangedRandom(angularVelocity.x,TWO*PI_DEFINE);
				newAngularVelocity.y = getRangedRandom(angularVelocity.y,TWO*PI_DEFINE);
				newVelocity.x = cos(newAngularVelocity.x) * getRangedRandom(velocity.x, velocity.x + getRangedRandom(0,10));
				newVelocity.y = sin(newAngularVelocity.y) * getRangedRandom(velocity.y, velocity.y + getRangedRandom(0,10));

				particleVector[particleSpot]->init(PARTICLE_POINT_EMITTER_TEXTURE_ID,position,newVelocity,angularVelocity,color,size, 
					getRangedRandom(PARTICLE_POINT_EMITTER_MIN_TIME_TO_LIVE, PARTICLE_POINT_EMITTER_MAX_TIME_TO_LIVE));
				particleVector[particleSpot]->setRandomColor(true);
			}
		}

		if(emit == true && active == true && (((int32_t)particleVector.size()) < maxParticles))
		{
			particleVector.push_back(new ParticleC);
			int32_t particleSpot = particleVector.size() - 1;

			particleVector[particleSpot]->init(PARTICLE_POINT_EMITTER_TEXTURE_ID,position,velocity,angularVelocity,color,size,
				getRangedRandom(PARTICLE_POINT_EMITTER_MIN_TIME_TO_LIVE, PARTICLE_POINT_EMITTER_MAX_TIME_TO_LIVE));
			particleVector[particleSpot]->setRandomColor(true);
		}

		if(emit == false && (((int32_t)particleVector.size()) <= 0))
		{
			active = false;
		}

		currentTime = 0;
	}
}