/*************************************************
**Authors: Kevin Cartrette, Alex Lemke			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary:  Pick up manager handles processing	**
**objects that can be picked up by players.		**
*************************************************/

#include <windows.h>											
#include <stdio.h>												
#include <stdarg.h>												
#include <math.h>												
#include "gl\gl.h"												
#include "gl\glu.h"												
#include "gl\glut.h"
#include "baseTypes.h"
#include "ShapeDraw.h"
#include "collInfo.h"
#include "object.h"
#include "PickUp.h"
#include "field.h"
#include "fieldmanager.h"
#include "random.h"
#include "stateManager.h"
#include "inputmanager.h"
#include "SpriteManager.h"
#include "gamePlayVariables.h"

PickUpC::PickUpC(float_t initPosX,float_t initPosY,float_t initVelX,float_t initVelY,int32_t id)
{
	mEnabled = false;
	mPosition.x = initPosX;
	mPosition.y = initPosY;
	mVelocity.x = initVelX;
	mVelocity.y = initVelY;
}

PickUpC::PickUpC()
{
    //mEnabled = true;
    mCollInfo.shape = CollInfoC::SHAPE_CIRCLE;

	lifetime = 300;
	moveTowardsPlayer = -1;
};

PickUpC::~PickUpC()
{
};

void PickUpC::update(double64_t milliseconds)
{
	if(mEnabled){
		switch(StateManagerC::GetInstance()->getState())
		{
			case StateManagerC::GAMEPLAY:
				if(moveTowardsPlayer == -1){
					//Decrease Lifetime
					lifetime--;

					//Friction
					mVelocity.scaleVector(0.06f);
					if(mVelocity.getLength() < 0.01)
						mVelocity.scaleVector(0.0f);
				}

				//Increase Position
				mPosition.x += mVelocity.x*milliseconds;
				mPosition.y += mVelocity.y*milliseconds;

				//Set Sprite Position
				mySprite.setPosition(mPosition.x, mPosition.y, 0.0f);
				DoCollisions();
				break;
		}
	}
}

void PickUpC::DoCollisions()
{
	CollideField();
}

void PickUpC::CollideField(void)
{
	FieldC *field = FieldManagerC::GetInstance()->getFieldPtr();

	float_t rightSide = field->getPosition()->x+((float_t)field->getWidth()/2.0f);
	float_t leftSide = field->getPosition()->x-((float_t)field->getWidth()/2.0f);
	float_t topSide = field->getPosition()->y+((float_t)field->getHeight()/2.0f);
	float_t bottomSide = field->getPosition()->y-((float_t)field->getHeight()/2.0f);
	
	//Box collision
	if(mPosition.x- mRadius <= leftSide  )
	{
		mVelocity.x = -mVelocity.x;
		mPosition.x = leftSide + mRadius;
		setRandomColor();
	}
	if(mPosition.x+ mRadius >= rightSide  )
	{
		mVelocity.x = -mVelocity.x;
		mPosition.x = rightSide - mRadius;
		setRandomColor();
	}
	if(mPosition.y- mRadius <= bottomSide  )
	{
		mVelocity.y = -mVelocity.y;
		mPosition.y = bottomSide + mRadius;
		setRandomColor();
	}
	if(mPosition.y+ mRadius >= topSide  )
	{
		mVelocity.y = -mVelocity.y;
		mPosition.y = topSide - mRadius;
		setRandomColor();
	}
};

void PickUpC::render()
{
	if(mEnabled){
		//Push Matrix
		glPushMatrix();

		//Pick Up begins flashing when it's about to disapear
		if((lifetime > 150) || (lifetime%5 == 0))
			DrawCircle(0.8f, mPosition.x, mPosition.y, 0,1,0.2f, false);	
		//SpriteManagerC::GetInstance()->drawSprite(mySprite);

		//Pop Matrix
		glPopMatrix();
	}

}

void PickUpC::setRandomColor()
{
	mPickUpColor = getRangedRandom(0,256);
	mPickUpColor += getRangedRandom(0,256) << 8;
	mPickUpColor += getRangedRandom(0,256) << 16;
}


bool PickUpC::getEnabled()
{
	return(mEnabled);
}

int32_t PickUpC::returnLifetime()
{
	return(lifetime);
}

//Move Towards Player
void PickUpC::setMoveTowarsPlayer(int32_t newMoveTowardsPlayer)
{
	moveTowardsPlayer = newMoveTowardsPlayer;
}

int32_t PickUpC::returnMoveTowardsPlayer()
{
	return(moveTowardsPlayer);
}