/*************************************************
**Authors: Kevin Cartrette, Alex Lemke			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary:  Pick up manager handles processing	**
**objects that can be picked up by players.		**
*************************************************/

#define PickUp_MANAGER_CPP
#include <assert.h>
#include <windows.h>											
#include <stdio.h>												
#include <stdarg.h>												
#include <math.h>											
#include "gl\gl.h"												
#include "gl\glu.h"												
#include "gl\glut.h"
#include "baseTypes.h"
#include "openglframework.h"	
#include "collinfo.h"
#include "object.h"
#include "PickUp.h"
#include "PickUpManager.h"
#include "random.h"
#include "SpriteManager.h"
#include "PickUp.h"
#include "PlayerManager.h"
#include "CameraManager.h"

PickUpManagerC* PickUpManagerC::sInstance=NULL;

PickUpManagerC *PickUpManagerC::CreateInstance()
{
	assert(sInstance==NULL);
	sInstance = new PickUpManagerC();
	return sInstance;
}

void PickUpManagerC::init()
{
	printf("Hey initing the PickUp manager!\n");
}

void PickUpManagerC::updatePickUps(double64_t milliseconds)
{
    //Regular Updates
	for (pickUpIterator = pickUpList.begin(); pickUpIterator != pickUpList.end();)
	{
		if((*pickUpIterator)->returnLifetime() > 0){ // If the pick up  is alive
			(*pickUpIterator)->update(milliseconds);

			Vector3C pickUpPosition = *((*pickUpIterator)->getPosition());
			float_t distanceToPickUp = pickUpPosition.distance((PlayerManagerC::GetInstance()->getPlayerPosition(0)));
			int32_t playerNumber = (*pickUpIterator)->returnMoveTowardsPlayer();
			if(playerNumber != -1)
			{
				Vector3C directionTowardsPlayer = (PlayerManagerC::GetInstance()->getPlayerPosition(playerNumber));
				directionTowardsPlayer.subtractVector(pickUpPosition);
				directionTowardsPlayer.normalize();
				directionTowardsPlayer.scaleVector(0.95f);
				(*pickUpIterator)->setVelocity(&directionTowardsPlayer);
			}

			if(distanceToPickUp < 10){ //Pick up has been collected
				//Increase the player's multiplier
				PlayerManagerC::GetInstance()->getPlayerPtr(0)->increaseMultiplier(1);

				//Remove Pick Up
				delete (*pickUpIterator);
				pickUpIterator = pickUpList.erase(pickUpIterator);
			}
			else	
			{
				//If the pick up is close to the player begin moving towards the player
				if(distanceToPickUp < 200)
					(*pickUpIterator)->setMoveTowarsPlayer(0);
					
				//Next pick up
				pickUpIterator++;
			}
		}
		else // else kill the pick up
		{
			//Remove Pick Up
			delete (*pickUpIterator);
			pickUpIterator = pickUpList.erase(pickUpIterator);
		}
	}
}

PickUpC *PickUpManagerC::getPickUpPtr(int32_t index)
{
	return pickUpList[index];
}

Vector3C PickUpManagerC::getPickUpPosition(int32_t index)
{
	return *(pickUpList[index]->getPosition());
}

void PickUpManagerC::renderPickUps()
{
	//Push Matrix
	glPushMatrix();

	//Draw every pick up
    for (pickUpIterator = pickUpList.begin(); pickUpIterator != pickUpList.end(); pickUpIterator++)
	{
		if(CameraManagerC::GetInstance()->checkIfInViewport(*((*pickUpIterator)->getPosition()))) //Only draw if it's in the current camera view
			(*pickUpIterator)->render();
	}

	//Pop Matrix
	glPopMatrix();
}

void PickUpManagerC::reset()
{
	for (pickUpIterator = pickUpList.begin(); pickUpIterator != pickUpList.end();)
    {
		delete(*pickUpIterator);
		pickUpIterator = pickUpList.erase(pickUpIterator);
	}
}

void PickUpManagerC::shutdown()
{
	for (pickUpIterator = pickUpList.begin(); pickUpIterator != pickUpList.end(); pickUpIterator++)
    {
		delete (*pickUpIterator);
	}
	//free(PickUpPtrs);
}

//Create a new pick up
void PickUpManagerC::createNewPickup(Vector3C newPosition, Vector3C newVelocity)
{
	PickUpC* newPickUp = new PickUpC();
	(*newPickUp).setPosition(&newPosition);
	(*newPickUp).setVelocity(&newVelocity);
	pickUpList.push_back(newPickUp);
}