/*************************************************
**Authors: Kevin Cartrette, Alex Lemke			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary: Player manager which handles all		**
**players movements and information.			**
*************************************************/

#define PLAYER_MANAGER_CPP
#include <assert.h>
#include <windows.h>											
#include <stdio.h>												
#include <stdarg.h>	
#include <math.h>	
#include "gl\gl.h"	
#include "gl\glu.h"												
#include "gl\glut.h"
#include "baseTypes.h"
#include "openglframework.h"	
#include "collinfo.h"
#include "object.h"
#include "player.h"
#include "PlayerManager.h"
#include "random.h"
#include "SpriteManager.h"
#include "SoundManager.h"
#include "EnemyManager.h"
#include "PickUpManager.h"
#include "FieldManager.h"
#include "ParticleManager.h"
#include "stateManager.h"

PlayerManagerC* PlayerManagerC::sInstance=NULL;

PlayerManagerC *PlayerManagerC::CreateInstance()
{
	assert(sInstance==NULL);
	sInstance = new PlayerManagerC();
	return sInstance;
}

void PlayerManagerC::init()
{
	printf("Hey initing the player manager!\n");
	playerPtrs = (PlayerC**)malloc(NUM_PLAYERS*sizeof(PlayerC*));
	for(uint32_t i=0;i<NUM_PLAYERS;++i)
    {
		playerPtrs[i] = new PlayerC(0.0f, 0.0f, getRangedRandom(-1.0f,1.0f),getRangedRandom(-1.0f,1.0f),i);
		playerPtrs[i]->setRandomColor();
        playerPtrs[i]->setRadius(10.0f);//getRangedRandom(10.0f,150.0f));
        playerPtrs[i]->enable();
		playerPtrs[i]->setID(i);
    }
	playerPtrs[0]->enable();

}

void PlayerManagerC::updatePlayers(DWORD milliseconds)
{
	switch(StateManagerC::GetInstance()->getState())
	{
		case StateManagerC::MAIN_MENU:
			break;
		case StateManagerC::GAMEPLAY:
			for(uint32_t i=0;i<NUM_PLAYERS;++i)
			{
				playerPtrs[i]->update(milliseconds);
				bool playerHitEnemy = playerPtrs[i]->DoCollisions();
				//If player hit an enemy then play death sound and reset game status.
				if(playerHitEnemy == true)
				{
					SoundManagerC::GetInstance()->playSoundEffect(SoundManagerC::die1,1.0f);
					EnemyManagerC::GetInstance()->reset();
					PickUpManagerC::GetInstance()->reset();
					ParticleManagerC::GetInstance()->shutdown();
					int32_t livesLeft = playerPtrs[i]->subtractLife();
					playerPtrs[i]->death();
					playerPtrs[i]->setInvincible(true);
					Vector3C position((*playerPtrs[i]->getPosition()).x, (*playerPtrs[i]->getPosition()).y, 0.0f);
					Vector3C velocity(CIRCLE_EMITTER_VELOCITY,CIRCLE_EMITTER_VELOCITY,0);
					Vector3C angularVelocity(CIRCLE_EMITTER_ANGULAR_VELOCITY,CIRCLE_EMITTER_ANGULAR_VELOCITY,0);
					FieldManagerC::GetInstance()->applyPush(playerPtrs[i]->getPosition(), 50, 0.02f, &temp);

					//Getting a random color
					temp.x = getRangedRandom(PARTICLE_RANDOM_COLOR_MIN_RED,PARTICLE_RANDOM_COLOR_MAX_RED);
					temp.y = getRangedRandom(PARTICLE_RANDOM_COLOR_MIN_GREEN,PARTICLE_RANDOM_COLOR_MAX_GREEN);
					temp.z = getRangedRandom(PARTICLE_RANDOM_COLOR_MIN_BLUE,PARTICLE_RANDOM_COLOR_MAX_BLUE);

					ParticleManagerC::GetInstance()->newEmitter(ParticleManagerC::CircleEmitter,CIRCLE_EMITTER_TIME_UPDATE,CIRCLE_EMITTER_TIME_TO_LIVE,position,velocity,angularVelocity,1.1,
						CIRCLE_EMITTER_MIN_PARTICLES,CIRCLE_EMITTER_MAX_PARTICLES, &temp);
		   
					//Apply a force to the grid.
					FieldManagerC::GetInstance()->applyPush((playerPtrs[i]->getPosition()), 1000, (-0.04f), &temp);

					//If player have zero lives left then play sound effect, switched to score menu and reset player.
					if(livesLeft <= 0)
					{
						SoundManagerC::GetInstance()->playSoundEffect(SoundManagerC::gameover, 1.0f);
						StateManagerC::GetInstance()->setState(StateManagerC::SCORE_MENU);
						//playerPtrs[i]->reset();
					}
				}
			}
			break;
    }
}

PlayerC *PlayerManagerC::getPlayerPtr(int32_t index)
{
	return playerPtrs[index];
}

Vector3C PlayerManagerC::getPlayerPosition(int32_t index)
{
	return *(playerPtrs[index]->getPosition());
}

void PlayerManagerC::renderPlayers()
{
	//Push Matrix
	glPushMatrix();

    for(uint32_t i=0;i<NUM_PLAYERS;++i)
    {
       playerPtrs[i]->render();
    }

	//Pop Matrix
	glPopMatrix();
}

void PlayerManagerC::shutdown()
{
	for(uint32_t i=0;i<NUM_PLAYERS;++i)
    {
		delete playerPtrs[i];
	}
	free(playerPtrs);
}