/*************************************************
**Authors: Kevin Cartrette, Alex Lemke			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary:  Enemy class with modifed render and **
**update to branch from standard enemy behavior.**
*************************************************/

#include "ProtonEnemy.h"
#include <Windows.h>
#include <stdio.h>
#include <stdlib.h>
#include "stateManager.h"
#include "gl\glu.h"	
#include "ShapeDraw.h"
#include <time.h>
#include "Vector2.h"
#include "SpriteManager.h"
#include "gamePlayVariables.h"
#include "FieldManager.h"

void ProtonEnemyC::update(double64_t milliseconds)
{
	if(mEnabled){
		switch(StateManagerC::GetInstance()->getState())
		{
			case StateManagerC::GAMEPLAY:
				Vector3C *playerPosition = getPlayerPosition();
				FieldManagerC::GetInstance()->applyColor(&mPosition, 10, &enemyColor);

				Vector3C velocityChange;
				float_t differenceX = playerPosition->x - mPosition.x;
				float_t differenceY = playerPosition->y - mPosition.y;
				velocityChange.x = differenceX + (mVelocity.x * 0.05f);
				velocityChange.y = differenceY + (mVelocity.y * 0.05f);

				velocityChange.normalize();
				velocityChange.scaleVector(0.065f);

				//Friction
				mVelocity.scaleVector(0.95f);
				mVelocity.add2DQuickVector(velocityChange);

				//Increase Position
				mPosition.x += mVelocity.x*milliseconds;
				mPosition.y += mVelocity.y*milliseconds;
				
#if(ENEMY_TEXTURES)
				enemySprite.setPosition(mPosition.x, mPosition.y, 0);
#endif
				DoCollisions();
				break;
		}
	}
	//printf("Updating with dart enemy class!\n");
}

void ProtonEnemyC::init(float_t initPosX, float_t initPosY, float_t initVelX, float_t initVelY, int32_t id)
{

	mPosition.x = initPosX;
	mPosition.y = initPosY;
	mVelocity.x = initVelX;
	mVelocity.y = initVelY;

	mRadius = ENEMY_RADIUS;
	scoreValue = PROTON_SCORE;
	health = 1;

	enemyColor.x = 0.4f;
	enemyColor.y = 0.4f;
	enemyColor.z = 1.0f;

#if(ENEMY_TEXTURES)
	enemySprite.setUp(5,0,0,0,64 * 2,64 * 2,0,0,0,64,64);
	enemySprite.setCentered(true);
#endif
}

void ProtonEnemyC::render()
{
	if(mEnabled){
		//printf("rendering with dart enemy class!\n");
		glPushMatrix();
#if(ENEMY_TEXTURES)
		//SpriteManagerC::GetInstance()->drawSprite(enemySprite);
		DrawCircle(8, mPosition.x,mPosition.y,0.05f,0.5f,1.0f,false);
#else
		char8_t red = (char8_t)((mEnemyColor>>16)& 0xFF);
		char8_t green = (char8_t)((mEnemyColor>>8) & 0xFF);
		char8_t blue = (char8_t)((mEnemyColor>>0) & 0xFF);
		bool filledVal;

		filledVal = true;

		DrawCircle(mRadius, mPosition.x,mPosition.y,red,green,blue,filledVal);
#endif

		//SpriteManagerC::GetInstance()->drawSprite();

		//Pop Matrix
		glPopMatrix();
	}
}