/*************************************************
**Authors: Kevin Cartrette, Alex Lemke			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary: Shape drawing helper functions.		**
*************************************************/

#include <windows.h>
#include "baseTypes.h"
#include "gl\gl.h"
#include "gl\glu.h"
#include "gl\glut.h"
#include <math.h>
#include <stdio.h>
#include "Vector3.h"
#include "gamePlayVariables.h"

void DrawCircle(float_t radius, float_t x, float_t y, char8_t r, char8_t g, char8_t b, bool filled)
{
	glEnable(GL_POINT_SMOOTH);

	glDisable(GL_TEXTURE_2D);
	glColor4ub(r, g, b,0x20);
	// Set the point size
	glPointSize(1.0);
	glBegin(GL_POINTS);
	float_t radiusMax = radius + HALF;
	glEnd();

	glColor4ub(r, g, b,0xFF);
 
	glPointSize(radius/HALF);
	glBegin(GL_POINTS);
	glVertex2f(x, y);
	glEnd();
}

void DrawLine(float_t startX, float_t startY, float_t endX, float_t endY, char8_t r, char8_t g, char8_t b)
{
	glColor3ub(r, g, b);
	// Draw filtered lines
	glEnable(GL_LINE_SMOOTH);

	glBegin(GL_LINE_STRIP);
		glVertex2f(startX, startY);
		glVertex2f(endX, endY);
	glEnd();
}

void DrawCircle(float_t radius, float_t x, float_t y, float_t r, float_t g, float_t b, bool filled)
{
	glEnable(GL_POINT_SMOOTH);
	glDisable(GL_TEXTURE_2D);
	glColor3f(r, g, b);
	if(!filled)
	{
		glPointSize(1.0);
		glBegin(GL_LINE_LOOP);
		float_t radiusMax = radius + 2.0f;
		float_t radiusMin = radius - 2.0f;
		for (int32_t i = 0; i < DEGREES_OF_CIRCLE; i += 3)
		{
			float_t degInRad = i*DEGREES_TO_RADIANS;
			glVertex2f(x + (cos(degInRad)*radius),y + (sin(degInRad)*radius));
			glVertex2f(x + (cos(degInRad)*radiusMax),y + (sin(degInRad)*radiusMax));
			glVertex2f(x + (cos(degInRad)*radiusMin),y + (sin(degInRad)*radiusMin));
		}
		glEnd();
	}
	else
	{
		//Drawing the Fill.
		glEnable(GL_BLEND); 
	    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); 
		glPointSize(radius/HALF);
		glBegin(GL_POINTS);
		glVertex2f(x, y);
		glEnd();
	}
}

void DrawLine(float_t startX, float_t startY, float_t endX, float_t endY, float_t r, float_t g, float_t b)
{
	glColor3f(r, g, b);
	// Draw filtered lines.
	glEnable(GL_LINE_SMOOTH);

	glBegin(GL_LINE_STRIP);
		glVertex2f(startX, startY);
		glVertex2f(endX, endY);
	glEnd();
}
void DrawCircle(float_t radius, float_t x, float_t y, Vector3C* color, bool filled)
{
	glEnable(GL_POINT_SMOOTH);
	glDisable(GL_TEXTURE_2D);
	glColor3f(color->x, color->y, color->y);
	if(!filled)
	{
		glPointSize(1.0);
		glBegin(GL_LINE_LOOP);
		float_t radiusMax = radius + 2.0f;
		float_t radiusMin = radius - 2.0f;
		for (int32_t i = 0; i < DEGREES_OF_CIRCLE; i += 3)
		{
			float_t degInRad = i*DEGREES_TO_RADIANS;
			glVertex2f(x + (cos(degInRad)*radius),y + (sin(degInRad)*radius));
			glVertex2f(x + (cos(degInRad)*radiusMax),y + (sin(degInRad)*radiusMax));
			glVertex2f(x + (cos(degInRad)*radiusMin),y + (sin(degInRad)*radiusMin));
		}
		glEnd();
	}
	else
	{
		//Drawing the Fill.
		glEnable(GL_BLEND); 
	    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); 
		glPointSize(radius/HALF);
		glBegin(GL_POINTS);
		glVertex2f(x, y);
		glEnd();
	}
}
void DrawLine(float_t startX, float_t startY, float_t endX, float_t endY, Vector3C* color)
{
	glColor3f(color->x, color->y, color->z);
	// Draw filtered lines.
	glEnable(GL_LINE_SMOOTH);

	glBegin(GL_LINE_STRIP);
		glVertex2f(startX, startY);
		glVertex2f(endX, endY);
	glEnd();
}