/*************************************************
**Authors: Kevin Cartrette, Alex Lemke			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary:  Basic sound manager class to handle **
**all sounds played inside the game.			**
*************************************************/

#include <assert.h>
#include <stdio.h>	
#include "SoundManager.h"
#include "InputManager.h"
#include "stateManager.h"

SoundManagerC* SoundManagerC::pInstance=NULL;

SoundManagerC *SoundManagerC::CreateInstance()
{
	assert(pInstance == NULL);
	pInstance = new SoundManagerC();
	return pInstance;
}

void SoundManagerC::init()
{
	result = FMOD::System_Create(&system);
	//Load sound.
	system->setOutput(FMOD_OUTPUTTYPE_DSOUND);
	system->init(MAX_NUMBER_SOUNDS, FMOD_INIT_NORMAL, NULL); 
	//Create the sound based on location, loop state and places inside the address of the sound array.
	//FMOD_LOOP_NORMAL will loop a sound over and over.
	result = system->createSound("Assets/Audio/Music/GeometryWars2.mp3", FMOD_LOOP_NORMAL, NULL, &sound[GeometryWars2]);
	//FMOD_DEFAULT will play a sound once then stop.
	result = system->createSound("Assets/Audio/SoundEffects/shotborn.wav", FMOD_DEFAULT, NULL, &sound[shotborn]);
	result = system->createSound("Assets/Audio/SoundEffects/Explo1.wav", FMOD_DEFAULT, NULL, &sound[Explo1]);
	result = system->createSound("Assets/Audio/SoundEffects/shotwall.wav", FMOD_DEFAULT, NULL, &sound[shotwall]);
	result = system->createSound("Assets/Audio/SoundEffects/die1.wav", FMOD_DEFAULT, NULL, &sound[die1]);
	result = system->createSound("Assets/Audio/SoundEffects/click.wav", FMOD_DEFAULT, NULL, &sound[click]);
	result = system->createSound("Assets/Audio/SoundEffects/gameover.wav", FMOD_DEFAULT, NULL, &sound[gameover]);
	result = system->createSound("Assets/Audio/SoundEffects/shield1.wav", FMOD_DEFAULT, NULL, &sound[shield1]);
	if(result == FMOD_OK){
		printf("Sound was made ok!!!\n");
	}

	//Set up channels and flag for pause for all sounds.
	for (int32_t i = Invalid_Sound + 1; i < Max_Sound; i++)
	{
		channel[i] = 0;
		isSoundPaused[i] = false;
	}
	
	//Play the temp level starting music.
	playSoundMusic(GeometryWars2, 0.1f);
}

void SoundManagerC::playSoundMusic(Sounds soundName, float_t volume)
{
	FMOD_RESULT result;
	//If sound channel is not playing then start playing.
	channel[soundName]->isPlaying(&isSoundPaused[soundName]);
	if(isSoundPaused[soundName] == false)
	{
		result = system->playSound(FMOD_CHANNEL_FREE, sound[soundName], false, &channel[soundName]);
		channel[soundName]->setVolume(volume);
		system->update();
	}
}

void SoundManagerC::playSoundEffect(Sounds soundName, float_t volume)
{
	FMOD_RESULT result;
	//If sound channel is not playing then start playing.
	channel[soundName]->isPlaying(&isSoundPaused[soundName]);
	if(isSoundPaused[soundName] == false)
	{
		result = system->playSound(FMOD_CHANNEL_FREE, sound[soundName], false, &channel[soundName]);
		channel[soundName]->setVolume(volume);
		system->update();
	}
}

void SoundManagerC::stopSound(Sounds soundName)
{
	channel[soundName]->stop();
}

void SoundManagerC::pauseSound(Sounds soundName)
{
	channel[soundName]->setPaused(true);
}

void SoundManagerC::unpauseSound(Sounds soundName)
{
	channel[soundName]->setPaused(false);
}

void SoundManagerC::setVolume(Sounds soundName, float_t volume)
{
	channel[soundName]->setVolume(volume);
	system->update();
}

void SoundManagerC::update()
{
	switch(StateManagerC::GetInstance()->getState())
	{
		case StateManagerC::GAMEPLAY:
			if(InputManagerC::GetInstance()->isLeftMouseButtonDown())
			{
				playSoundEffect(shotborn,1.0);
			}
			break;
		case StateManagerC::MAIN_MENU:
			if(InputManagerC::GetInstance()->currentKeys['W'] == (int32_t)true || InputManagerC::GetInstance()->currentKeys['S'] == (int32_t)true)
			{
				playSoundEffect(click,1.0);
			}
			break;
	}
}

void SoundManagerC::shutdown()
{
	for (int32_t i = Invalid_Sound + 1; i < Max_Sound; i++)
	{
		result = sound[i]->release();
	}
	result = system->release();
}