/*************************************************
**Authors: Kevin Cartrette, Alex Lemke			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary:  Basic sound manager class to handle **
**all sounds played inside the game.			**
*************************************************/

#ifndef SOUNDMANAGER_H
#define SOUNDMANAGER_H

#include "fmod.h"
#include "baseTypes.h"
#include "gamePlayVariables.h"
#include <fmod.hpp>

/*************************************************
**Summary: Sound manager class which handles	**
**processing all sound attributes.				**
**Authors: Kevin Cartrette, Alex Lemke			**
*************************************************/
class SoundManagerC
{
public:
	//Sound enums for all sounds.
	enum Sounds
	{
		Invalid_Sound,
		GeometryWars2,
		shotborn,
		Explo1,
		shotwall,
		die1,
		click,
		gameover,
		shield1,
		Max_Sound
	};

	static SoundManagerC *CreateInstance();
	static SoundManagerC *GetInstance() {return pInstance;};
	~SoundManagerC(){};
	void init();
	void update();
	//Music sounds have a different loop setting on creation, other wise same as sound effect.
	void playSoundMusic(Sounds soundName, float_t volume);
	//Sound effects play only once.
	void playSoundEffect(Sounds soundName, float_t volume);
	//Simple functions to control sound status.
	void stopSound(Sounds soundName);
	void pauseSound(Sounds soundName);
	void unpauseSound(Sounds soundName);
	void setVolume(Sounds soundName, float_t volume);
	void shutdown();
private:
	static SoundManagerC *pInstance;
	FMOD::System* system;
	//Max number of channels is 32 for now.
	FMOD::Channel* channel[MAX_NUMBER_SOUNDS];
	FMOD_RESULT  result;
	//Max number of sounds right now 32.
	FMOD::Sound* sound[MAX_NUMBER_SOUNDS];
	//Contain flag for each sound if true or false.
	bool isSoundPaused[MAX_NUMBER_SOUNDS];
};
#endif