/*************************************************
**Authors: Kevin Cartrette, Alex Lemke			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary:  Enemy class with modifed render and **
**update to branch from standard enemy behavior.**
*************************************************/

#include "SpinnerEnemy.h"
#include <Windows.h>
#include <stdio.h>
#include <stdlib.h>
#include "stateManager.h"
#include "gl\glu.h"	
#include "ShapeDraw.h"
#include <time.h>
#include "SpriteManager.h"
#include "gamePlayVariables.h"
#include "FieldManager.h"

void SpinnerEnemyC::update(double64_t milliseconds)
{
	if(mEnabled){
		switch(StateManagerC::GetInstance()->getState())
		{
			case StateManagerC::GAMEPLAY:
				FieldManagerC::GetInstance()->applyColor(&mPosition, 10, &enemyColor);
				mPosition.x += mVelocity.x*milliseconds;
				mPosition.y += mVelocity.y*milliseconds;

#if(ENEMY_TEXTURES)
				enemySprite.setPosition(mPosition.x, mPosition.y, 0);
#endif
				DoCollisions();
				break;
		}
	}
}

void SpinnerEnemyC::init(float_t initPosX, float_t initPosY, float_t initVelX, float_t initVelY, int32_t id)
{
	mPosition.x = initPosX;
	mPosition.y = initPosY;
	mVelocity.x = initVelX;
	mVelocity.y = initVelY;

	mRadius = ENEMY_RADIUS;
	health = 1;
	scoreValue = SPINNER_SCORE;

	enemyColor.x = 0.7f;
	enemyColor.y = 0.0f;
	enemyColor.z = 1.0f;

#if(ENEMY_TEXTURES)
	enemySprite.setUp(6,0,0,0,64 * 2,64 * 2,0,0,0,64,64);
	enemySprite.setCentered(true);
#endif
	direction = randomDirection();
}
void SpinnerEnemyC::render()
{
	if(mEnabled){
		//printf("rendering with dart enemy class!\n");
		glPushMatrix();
#if(ENEMY_TEXTURES)
		enemySprite.setRotation(enemySprite.returnRotation() + SPIN_RATE_EACH_FRAME);
		SpriteManagerC::GetInstance()->drawSprite(enemySprite);
#else
		char8_t red = (char8_t)((mEnemyColor>>16)& 0xFF);
		char8_t green = (char8_t)((mEnemyColor>>8) & 0xFF);
		char8_t blue = (char8_t)((mEnemyColor>>0) & 0xFF);
		bool filledVal;

		filledVal = true;

		DrawCircle(mRadius, mPosition.x,mPosition.y,red,green,blue,filledVal);
#endif
		//Pop Matrix
		glPopMatrix();

#if DRAW_COLLISIONS == 1
	DrawCircle(mRadius, mPosition.x, mPosition.y, 1.0f, 0.0f, 0.0f, false);
#endif
	}
}

int32_t SpinnerEnemyC::randomDirection()
{
	return (rand()%2);
}