/*************************************************
**Authors: Alex Lemke, Kevin Cartrette			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary: A sprite manager. Holds and draws    **
**		   the actual sprites.    				**
*************************************************/

#define STATE_MANAGER_CPP
#include <windows.h>										
#include <stdio.h>												
#include <stdarg.h>												
#include <math.h>											
#include "gl\gl.h"											
#include "gl\glu.h"											
#include "gl\glut.h"
#include <assert.h>
#include "baseTypes.h"
#include "openglframework.h"	
#include "spriteManager.h"
#include "random.h"
#include "stateManager.h"
#include "soil.h"

SpriteManagerC* SpriteManagerC::sInstance=NULL;

SpriteManagerC *SpriteManagerC::CreateInstance()
{
	if(sInstance!=NULL)return sInstance;
	else
	sInstance = new SpriteManagerC();
	return sInstance;
}
void SpriteManagerC::init(int height, int width)
{
	currentTextureNumber = -1;

	//Allocating Space
	spriteArray = (SpriteC*)malloc(sizeof(SpriteC) * 20);
	textureList = (TextureC*)malloc(sizeof(TextureC) * 20);

	//Background
	textureList[0].setUpTexture(SOIL_load_OGL_texture("Assets\\Textures\\geometryWarsBackground.png", SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID,
	SOIL_FLAG_MIPMAPS | SOIL_FLAG_INVERT_Y | SOIL_FLAG_NTSC_SAFE_RGB | SOIL_FLAG_COMPRESS_TO_DXT), 1000,1000); 

	//Player Ship
	textureList[1].setUpTexture(SOIL_load_OGL_texture("Assets\\Textures\\player.png", SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID,
	SOIL_FLAG_MIPMAPS | SOIL_FLAG_INVERT_Y | SOIL_FLAG_NTSC_SAFE_RGB | SOIL_FLAG_COMPRESS_TO_DXT), 64, 64); 

	//Cross Hair
	textureList[2].setUpTexture(SOIL_load_OGL_texture("Assets\\Textures\\CrossHair.png", SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID,
	SOIL_FLAG_MIPMAPS | SOIL_FLAG_INVERT_Y | SOIL_FLAG_NTSC_SAFE_RGB | SOIL_FLAG_COMPRESS_TO_DXT), 32, 32); 

	//Bullets
	textureList[3].setUpTexture(SOIL_load_OGL_texture("Assets\\Textures\\GeometryWarsBullet.png", SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID,
	SOIL_FLAG_MIPMAPS | SOIL_FLAG_INVERT_Y | SOIL_FLAG_NTSC_SAFE_RGB | SOIL_FLAG_COMPRESS_TO_DXT), 32, 32);

	//Simple Particle
	textureList[4].setUpTexture(SOIL_load_OGL_texture("Assets\\Textures\\particleSmall.png", SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID,
	SOIL_FLAG_MIPMAPS | SOIL_FLAG_INVERT_Y | SOIL_FLAG_NTSC_SAFE_RGB | SOIL_FLAG_COMPRESS_TO_DXT), 4, 4);

	//Chaser enemy
	textureList[5].setUpTexture(SOIL_load_OGL_texture("Assets\\Textures\\grunt.png", SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID,
	SOIL_FLAG_MIPMAPS | SOIL_FLAG_INVERT_Y | SOIL_FLAG_NTSC_SAFE_RGB | SOIL_FLAG_COMPRESS_TO_DXT), 64, 64);

	//Spinner enemy
	textureList[6].setUpTexture(SOIL_load_OGL_texture("Assets\\Textures\\wanderer.png", SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID,
	SOIL_FLAG_MIPMAPS | SOIL_FLAG_INVERT_Y | SOIL_FLAG_NTSC_SAFE_RGB | SOIL_FLAG_COMPRESS_TO_DXT), 64, 64);

	//Dart Enemy
	textureList[7].setUpTexture(SOIL_load_OGL_texture("Assets\\Textures\\rocket.png", SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID,
	SOIL_FLAG_MIPMAPS | SOIL_FLAG_INVERT_Y | SOIL_FLAG_NTSC_SAFE_RGB | SOIL_FLAG_COMPRESS_TO_DXT), 64, 64);

	//Dodge Enemy
	textureList[8].setUpTexture(SOIL_load_OGL_texture("Assets\\Textures\\weaver.png", SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID,
	SOIL_FLAG_MIPMAPS | SOIL_FLAG_INVERT_Y | SOIL_FLAG_NTSC_SAFE_RGB | SOIL_FLAG_COMPRESS_TO_DXT), 64, 64);

	//Black Hole Enemy
	textureList[9].setUpTexture(SOIL_load_OGL_texture("Assets\\Textures\\gravitywell.png", SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID,
	SOIL_FLAG_MIPMAPS | SOIL_FLAG_INVERT_Y | SOIL_FLAG_NTSC_SAFE_RGB | SOIL_FLAG_COMPRESS_TO_DXT), 64, 64);

	//GUI Icons
	textureList[10].setUpTexture(SOIL_load_OGL_texture("Assets\\Textures\\guiIcons.png", SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID,
	SOIL_FLAG_MIPMAPS | SOIL_FLAG_INVERT_Y | SOIL_FLAG_NTSC_SAFE_RGB | SOIL_FLAG_COMPRESS_TO_DXT), 64, 128);

	//Title Image
	textureList[11].setUpTexture(SOIL_load_OGL_texture("Assets\\Textures\\geoWars_Title.png", SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID,
	SOIL_FLAG_MIPMAPS | SOIL_FLAG_INVERT_Y | SOIL_FLAG_NTSC_SAFE_RGB | SOIL_FLAG_COMPRESS_TO_DXT), 712, 64);
}
void SpriteManagerC::shutdown()
{

}

void SpriteManagerC::drawSprite(SpriteC mySprite)
{
	int32_t newTextureNumber = mySprite.returnTextureID();
	
	//Setting Up Texture Drawing Mode
	glEnable(GL_TEXTURE_2D);
	if(currentTextureNumber != newTextureNumber){
		currentTextureNumber = newTextureNumber;
		glBindTexture (GL_TEXTURE_2D, textureList[currentTextureNumber].returnTexture());
	}

	//Push Matrix
	glPushMatrix();

	Vector3C quadPosition = mySprite.returnPosition();
	float quadWidth = mySprite.returnWidth();
	float quadHeight = mySprite.returnHeight();
	float quadRotation = mySprite.returnRotation();

	Vector2C texturePosition = mySprite.returnTexturePosition();
	float textureWidth = mySprite.returnTextureWidth();
	float textureHeight = mySprite.returnTextureHeight();
		
	float totalTextureWidth = textureList[currentTextureNumber].returnWidth();
	float totalTextureHeight = textureList[currentTextureNumber].returnHeight();
	
	
	glTranslatef(quadPosition.x, quadPosition.y, quadPosition.z);
	glRotatef(quadRotation, 0, 0, 1);
	
	//Push Matrix
	glPushMatrix();

	//Drawing the Quad
	glBegin (GL_QUADS);

	glColor4ub(0xFF, 0xFF, 0xFF,0xFF);
	if(mySprite.returnedCentered())
	{
		//Left Top
		glTexCoord2f ((texturePosition.x) / totalTextureWidth, (texturePosition.y) / totalTextureHeight);
		glVertex3f (- quadWidth/2, - quadHeight/2, 0);
		//Right Top
		glTexCoord2f ((texturePosition.x + textureWidth) / totalTextureWidth, (texturePosition.y) / totalTextureHeight);
		glVertex3f (+ quadWidth/2,  - quadHeight/2, 0);
		//Right Botton
		glTexCoord2f ((texturePosition.x + textureWidth) / totalTextureWidth, (texturePosition.y + textureHeight) / totalTextureHeight);
		glVertex3f ( + quadWidth/2,  + quadHeight/2, 0);
		//Left Bottom
		glTexCoord2f ((texturePosition.x) / totalTextureWidth, (texturePosition.y  + textureHeight) / totalTextureHeight);
		glVertex3f ( - quadWidth/2,  + quadHeight/2, 0);

	}
	else
	{
		//Left Top
		glTexCoord2f ((texturePosition.x) / totalTextureWidth, (texturePosition.y) / totalTextureHeight);
		glVertex3f (0, 0, 0);
		//Right Top
		glTexCoord2f ((texturePosition.x + textureWidth) / totalTextureWidth, (texturePosition.y) / totalTextureHeight);
		glVertex3f (quadWidth, 0, 0);
		//Right Botton
		glTexCoord2f ((texturePosition.x + textureWidth) / totalTextureWidth, (texturePosition.y + textureHeight) / totalTextureHeight);
		glVertex3f ( + quadWidth, + quadHeight, 0);
		//Left Bottom
		glTexCoord2f ((texturePosition.x) / totalTextureWidth, (texturePosition.y  + textureHeight) / totalTextureHeight);
		glVertex3f (0, + quadHeight, 0);
	}

	glEnd();

	//Pop Matrix
	glPopMatrix();

	//Pop Matrix
	glPopMatrix();

	glDisable(GL_TEXTURE_2D);

}

void SpriteManagerC::drawParticle(SpriteC mySprite, float_t red, float_t blue, float_t green, float_t alpha)
{
	int textureNumber = mySprite.returnTextureID();

	//Setting Up Texture Drawing Mode
	glEnable(GL_TEXTURE_2D);
	glBindTexture (GL_TEXTURE_2D, textureList[textureNumber].returnTexture());

	//Push Matrix
	glPushMatrix();

	Vector3C quadPosition = mySprite.returnPosition();
	float quadWidth = mySprite.returnWidth();
	float quadHeight = mySprite.returnHeight();
	float quadRotation = mySprite.returnRotation();

	Vector2C texturePosition = mySprite.returnTexturePosition();
	float textureWidth = mySprite.returnTextureWidth();
	float textureHeight = mySprite.returnTextureHeight();
		
	float totalTextureWidth = textureList[textureNumber].returnWidth();
	float totalTextureHeight = textureList[textureNumber].returnHeight();
	
	
	glTranslatef(quadPosition.x, quadPosition.y, quadPosition.z);
	glRotatef(quadRotation, 0, 0, 1);
	
	//Push Matrix
	glPushMatrix();

	//Drawing the Quad
	glBegin (GL_QUADS);

	//glColor4ub(0xFF, 0xFF, 0xFF,0xFF);
	glColor4f(red,green,blue,alpha);
	if(mySprite.returnedCentered())
	{
		//Left Top
		glTexCoord2f ((texturePosition.x) / totalTextureWidth, (texturePosition.y) / totalTextureHeight);
		glVertex3f (- quadWidth/2, - quadHeight/2, 0);
		//Right Top
		glTexCoord2f ((texturePosition.x + textureWidth) / totalTextureWidth, (texturePosition.y) / totalTextureHeight);
		glVertex3f (+ quadWidth/2,  - quadHeight/2, 0);
		//Right Botton
		glTexCoord2f ((texturePosition.x + textureWidth) / totalTextureWidth, (texturePosition.y + textureHeight) / totalTextureHeight);
		glVertex3f ( + quadWidth/2,  + quadHeight/2, 0);
		//Left Bottom
		glTexCoord2f ((texturePosition.x) / totalTextureWidth, (texturePosition.y  + textureHeight) / totalTextureHeight);
		glVertex3f ( - quadWidth/2,  + quadHeight/2, 0);

	}
	else
	{
		//Left Top
		glTexCoord2f ((texturePosition.x) / totalTextureWidth, (texturePosition.y) / totalTextureHeight);
		glVertex3f (0, 0, 0);
		//Right Top
		glTexCoord2f ((texturePosition.x + textureWidth) / totalTextureWidth, (texturePosition.y) / totalTextureHeight);
		glVertex3f (quadWidth, 0, 0);
		//Right Botton
		glTexCoord2f ((texturePosition.x + textureWidth) / totalTextureWidth, (texturePosition.y + textureHeight) / totalTextureHeight);
		glVertex3f ( + quadWidth, + quadHeight, 0);
		//Left Bottom
		glTexCoord2f ((texturePosition.x) / totalTextureWidth, (texturePosition.y  + textureHeight) / totalTextureHeight);
		glVertex3f (0, + quadHeight, 0);
	}

	glEnd();

	//Pop Matrix
	glPopMatrix();

	//Pop Matrix
	glPopMatrix();

	glDisable(GL_TEXTURE_2D);
}