/*************************************************
**Authors: Kevin Cartrette, Alex Lemke			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary: Helper Vector3 class.				**
*************************************************/

#include <windows.h>											// Header File For Windows
#include "baseTypes.h"
#include "Vector3.h"
#include <math.h>												// Header File For Math Operations
#include "gamePlayVariables.h"


Vector3C::Vector3C()
{
	x = 0.0f;
	y = 0.0f;
	z = 0.0f;
}

Vector3C::Vector3C(float_t newX, float_t newY)
{
	x = newX;
	y = newY;
	z = 0.0f;
}

Vector3C::Vector3C(float_t newX, float_t newY, float_t newZ)
{
	x = newX;
	y = newY;
	z = newZ;
}

void Vector3C::addVector(Vector3C a)
{
	x += a.x;
	y += a.y;
	z += a.z;
}

void Vector3C::subtractVector(Vector3C a)
{
	x -= a.x;
	y -= a.y;
	z -= a.z;
}
//Quick Math
void Vector3C::add2DQuickVector(Vector3C a)
{
	x += a.x;
	y += a.y;
}

void Vector3C::subtract2DQuickVector(Vector3C a)
{
	x -= a.x;
	y -= a.y;
}

Vector3C Vector3C::crossProduct(Vector3C a, Vector3C b)
{
	Vector3C result;

	result.x = ((a.y * b.z) - (a.z * b.y));
	result.y = ((a.z * b.x) - (a.x * b.z));
	result.z = ((a.x * b.y) - (a.y * b.x));

	return(result);
}

void Vector3C::setEqualToVector(Vector3C a)
{
	x = a.x;
	y = a.y;
	z = a.z;
}

float_t Vector3C::dotProduct(Vector3C b)
{
	float_t result = ((this->x * b.x) + (this->y * b.y) + (this->z * b.z));
	return(result);
}

float_t Vector3C::dotProduct(Vector2C b)
{
	float_t result = ((this->x * b.x) + (this->y * b.y));
	return(result);
}

float_t Vector3C::distance(Vector3C b)
{
	float_t returnValue = sqrtf(powf((b.x - this->x), HALF) + powf((b.y - this->y), HALF) + powf((b.z - this->z), HALF));
	return(returnValue);
}

void Vector3C::normalize()
{
	float length = sqrt(powf(x, UNIT_CIRCLE_MODIFER) + powf(y, UNIT_CIRCLE_MODIFER));
	x /= length;
	y /= length;
}

float_t Vector3C::getLength()
{
	return(sqrt(powf(x, UNIT_CIRCLE_MODIFER) + powf(y, UNIT_CIRCLE_MODIFER)));
}

void Vector3C::scaleVector(float_t scaleAmount)
{
	x *= scaleAmount;
	y *= scaleAmount;
	z *= scaleAmount;
}

float_t Vector3C::angleBetween(Vector3C a)
{
	return(acos(dotProduct(a)));
}

void Vector3C::addXComponet(float_t xAmount)
{
	x += xAmount;
}
void Vector3C::addYComponet(float_t yAmount)
{
	y += yAmount;
}

void Vector3C::addZComponet(float_t yAmount)
{
	y += yAmount;
}

void Vector3C::rotateAroundZAxis(float_t angle)
{
	x = (x * cosf(angle)) + (-y * sinf(angle));
	y = (x * sinf(angle)) + (y * cosf(angle));
}

void Vector3C::addVectorWithScale(Vector3C a, float_t scale)
{
	x += (a.x * scale);
	y += (a.y * scale);
	z += (a.z * scale);
}
void Vector3C::setValues(float_t newX, float_t newY)
{
	x = newX;
	y = newY;
	z = 0.0f;
}

void  Vector3C::setValues(float_t newX, float_t newY, float_t newZ)
{
	x = newX;
	y = newY;
	z = newZ;
}