/*************************************************
**Authors: Kevin Cartrette, Alex Lemke			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary:  Enemy class which is the base for   **
**all enemies classes.							**
*************************************************/

#include <windows.h>									
#include <stdio.h>											
#include <stdarg.h>									
#include <math.h>											
#include "gl\gl.h"										
#include "gl\glu.h"											
#include "gl\glut.h"
#include "baseTypes.h"
#include "ShapeDraw.h"
#include "collInfo.h"
#include "object.h"
#include "enemy.h"
#include "field.h"
#include "fieldmanager.h"
#include "random.h"
#include "stateManager.h"
#include "inputmanager.h"
#include "PlayerManager.h"
#include "SpriteManager.h"
#include "player.h"
#include "Bullet.h"
#include "gamePlayVariables.h"

EnemyC::EnemyC(float_t initPosX,float_t initPosY,float_t initVelX,float_t initVelY,int32_t id)
{
	mPosition.x = initPosX;
	mPosition.y = initPosY;
	mVelocity.x = initVelX;
	mVelocity.y = initVelY;
}

EnemyC::EnemyC()
{
    mEnabled = true;
    mCollInfo.shape = CollInfoC::SHAPE_CIRCLE;
}

EnemyC::~EnemyC()
{
}

void EnemyC::update(double64_t milliseconds)
{
	if(mEnabled){
		switch(StateManagerC::GetInstance()->getState())
		{
			case StateManagerC::GAMEPLAY:
				mPosition.x += mVelocity.x*milliseconds/10;
				mPosition.y += mVelocity.y*milliseconds/10;
				DoCollisions();
				break;
		}
	}
}
void EnemyC::DoCollisions()
{
	CollideField();
}

//Collision functions
bool EnemyC::checkIfBulletsCollide(int32_t playerNumber)
{
	if(this->mEnabled){
		PlayerC *temporaryPlayerPtr = PlayerManagerC::GetInstance()->getPlayerPtr(playerNumber);
		if(temporaryPlayerPtr->getEnabled()) // If the player is enabled
		{
			for(int32_t b = 0; b < temporaryPlayerPtr->getNumBullets(); b++)
			{
				BulletC *temporaryBulletPtr = temporaryPlayerPtr->getBulletPtr(b);
				if(temporaryBulletPtr->getEnabled()) // If the bullet is enabled
				{
					Vector3C bulletPosition = (*(temporaryBulletPtr->getPosition()));
					if(checkIfCollideRadial((mRadius + temporaryBulletPtr->getRadius()), bulletPosition))
					{
						//Bullet has collided so destroy it
						temporaryBulletPtr->disable();
						return(true);
					}
				}
			}
		}
	}
	return(false);
}

bool EnemyC::checkIfCollideRadial(double64_t radius, Vector3C position)
{
	if(mPosition.distance(position) < (mRadius + radius))
	{
		return(true);
	}
	return(false);
}

bool EnemyC::checkIfCollideRadial(double64_t playerRadius, Vector3C playerPosition, double64_t enemyRadius, Vector3C enemyPosition)
{
	if(playerPosition.distance(enemyPosition) < (enemyRadius + playerRadius))
	{
		return(true);
	}

	return(false);
}

void EnemyC::CollideField(void)
{
	FieldC *field = FieldManagerC::GetInstance()->getFieldPtr();

	float_t rightSide = field->getPosition()->x+((float_t)field->getWidth()/2.0f);
	float_t leftSide = field->getPosition()->x-((float_t)field->getWidth()/2.0f);
	float_t topSide = field->getPosition()->y+((float_t)field->getHeight()/2.0f);
	float_t bottomSide = field->getPosition()->y-((float_t)field->getHeight()/2.0f);
	
	//Box collision
	if(mPosition.x - mRadius <= leftSide  )
	{
		mVelocity.x = -mVelocity.x;
		mPosition.x = leftSide + mRadius;
		setRandomColor();
	}
	if(mPosition.x + mRadius >= rightSide  )
	{
		mVelocity.x = -mVelocity.x;
		mPosition.x = rightSide - mRadius;
		setRandomColor();
	}
	if(mPosition.y - mRadius <= bottomSide  )
	{
		mVelocity.y = -mVelocity.y;
		mPosition.y = bottomSide + mRadius;
		setRandomColor();
	}
	if(mPosition.y + mRadius >= topSide  )
	{
		mVelocity.y = -mVelocity.y;
		mPosition.y = topSide - mRadius;
		setRandomColor();
	}
}

void EnemyC::render()
{
	if(mEnabled){
		//Push Matrix
		glPushMatrix();

		char8_t red = (char8_t)((mEnemyColor>>16)& 0xFF);
		char8_t green = (char8_t)((mEnemyColor>>8) & 0xFF);
		char8_t blue = (char8_t)((mEnemyColor>>0) & 0xFF);
		bool filledVal;

		filledVal = true;

		DrawCircle(mRadius,mPosition.x,mPosition.y,red,green,blue,filledVal);


		//SpriteManagerC::GetInstance()->drawSprite();

		//Pop Matrix
		glPopMatrix();
	}
}

void EnemyC::init(float_t initPosX,float_t initPosY,float_t initVelX,float_t initVelY,int32_t id)
{
}

void EnemyC::setRandomColor()
{
	mEnemyColor = getRangedRandom(0,256);
	mEnemyColor += getRangedRandom(0,256) << 8;
	mEnemyColor += getRangedRandom(0,256) << 16;
}

Vector3C* EnemyC::getPlayerPosition()
{
	PlayerC* player = PlayerManagerC::GetInstance()->getPlayerPtr(0);
	return player->getPosition();
}

//Score
uint32_t EnemyC::getScoreValue()
{
	return scoreValue;
}

//Health
int32_t EnemyC::getHealth()
{
	return(health);
}

void EnemyC::decreaseHealth()
{
	health--;
}

//Pushing/Pulling Enemies
void EnemyC::applyForce(Vector3C pushDirection)
{
	//Apply the force to the enemies velocity
	mVelocity.add2DQuickVector(pushDirection);
	//We don't want things going to fast due to the black hole
	if(mVelocity.getLength() >= ENEMY_MAX_SPEED)
	{
		mVelocity.normalize();
		mVelocity.scaleVector((float_t)ENEMY_MAX_SPEED);
	}
}

bool EnemyC::checkIfPlayerPositionCollide(double64_t radius, int32_t playerNumber)
{
	if(this->mEnabled){
		Vector3C position = *PlayerManagerC::GetInstance()->getPlayerPtr(playerNumber)->getPosition();
		if(this->checkIfCollideRadial(radius, position))
			return(true);
	}
	return(false);
}