/*************************************************
**Authors: Kevin Cartrette, Alex Lemke			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary:  Enemy class which is the base for   **
**all enemies classes.							**
*************************************************/

#ifndef ENEMY_H
#define ENEMY_H

#include "Sprite.h"
#include "Object.h"
#include "baseTypes.h"

class Vector3C;

/*************************************************
**Summary: Enemy class which sets the standard  **
**for all enemies.								**
**Authors: Kevin Cartrette, Alex Lemke			**
*************************************************/
class EnemyC : public ObjectC
{
public:
	EnemyC();
	EnemyC(float_t initPosX, float_t initPosY, float_t initVelX, float_t initVelY,int32_t id);
	~EnemyC();
	//Function prototypes.
	void setEnemyColor(int32_t color) {mEnemyColor = color;};
	int32_t getEnemyColor() {return mEnemyColor;};
	//Virtual functions to handle updating and rendering of enemy classes.
	virtual void update(double64_t milliseconds);
	virtual void render() ;
	virtual void init(float_t initPosX, float_t initPosY, float_t initVelX, float_t initVelY,int32_t id);
	void DoCollisions();
	void setRadius(float_t radius) {mCollInfo.param1 = radius;mRadius = radius;};
	void setRandomColor();
	void reduceVelocity(float_t factor);
	virtual void setDirection(int32_t setDirection){};
	Vector3C* getPlayerPosition();
	uint32_t getScoreValue();
	//functions for Collisions
	virtual bool checkIfBulletsCollide(int32_t playerNumber);
	virtual bool checkIfPlayerPositionCollide(double64_t radius, int32_t playerNumber);
	//Determines if the circle collides with the enemies circle
	bool checkIfCollideRadial(double64_t radius, Vector3C position);
	bool checkIfCollideRadial(double64_t playerRadius, Vector3C playerPosition, double64_t enemyRadius, Vector3C enemyPosition);
	//Health
	int32_t getHealth();
	virtual void decreaseHealth();
	//Pulling Enemies
	virtual void applyForce(Vector3C pushDirection);
protected:
	//Enemy sprite.
	SpriteC enemySprite;
	void CollideField();
	int32_t mEnemyColor;
	uint32_t scoreValue;
	int32_t health;
	float_t mRadius;
	bool mAtRest;
	int32_t direction;

	//Enemy Color to blend with the grid
	Vector3C enemyColor;
};
#endif