/*************************************************
**Authors: Alex Lemke, Kevin Cartrette			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary: The actual geometry wars grid        **
*************************************************/

#include <windows.h>											// Header File For Windows
#include <stdio.h>												// Header File For Standard Input / Output
#include <stdarg.h>												// Header File For Variable Argument Routines
#include <math.h>												// Header File For Math Operations
#include "gl\gl.h"												// Header File For The OpenGL32 Library
#include "gl\glu.h"												// Header File For The GLu32 Library
#include "gl\glut.h"
#include "baseTypes.h"
#include "collInfo.h"
#include "object.h"
#include "field.h"
#include "ShapeDraw.h"
#include "StateManager.h"
#include "Sprite.h"
#include "SpriteManager.h"
#include "GridPoint.h"
#include "CameraManager.h"
#include "gamePlayVariables.h"

void FieldC::moveUp()
{
	int y;
	y=5;
}
FieldC::FieldC(float_t x, float_t y, uint32_t height, uint32_t width, uint32_t initColor)
{
    mEnabled = true;
    mFieldColor = initColor;

	mWidth = width;
	mHeight = height;
	mPosition.x = 0;
	mPosition.y = 0;

	mySprite.setCentered(true);
	mySprite.setUp(0, 0.0f, -1000.0f, 0.0f, width, height, 0.0f, 0.0f, 0.0f, 480.0f, 640.0f);

	//Gettign Edge Positions
	float_t left = mPosition.x - (float_t)mWidth/2.0f;
	float_t right = mPosition.x + (float_t)mWidth/2.0f;
	float_t bottom = mPosition.y - (float_t)mHeight/2.0f;
	float_t top = mPosition.y + (float_t)mHeight/2.0f;

	numberOfGridPoints = NUMBER_OF_GRIDPOINTS;
	//Set Up Grid
	gridPoints = (GridPointC***)malloc(numberOfGridPoints * sizeof(GridPointC**));
	
	Vector3C newPosition;
	for(int32_t i = 0; i < numberOfGridPoints; i++)
	{
		gridPoints[i] = (GridPointC**)malloc(numberOfGridPoints * sizeof(GridPointC*));
		//gridPoints[i]
		for(int32_t j = 0; j < numberOfGridPoints; j++)
		{
			gridPoints[i][j] = new GridPointC;
			newPosition.setValues(((float_t)i / (float_t)numberOfGridPoints) * width - width/2 + (width/numberOfGridPoints/2), ((float_t)j / (float_t)numberOfGridPoints) * height - height/2  + (height/numberOfGridPoints/2));
			gridPoints[i][j]->setPosition(newPosition);
			if(i == 0)
			{
				gridPoints[i][j]->setLocked(true);
				newPosition.setValues(left, newPosition.y);
				gridPoints[i][j]->setPosition(newPosition);
			}
			else if(i == numberOfGridPoints - 1)
			{
				gridPoints[i][j]->setLocked(true);
				newPosition.setValues(right, newPosition.y);
				gridPoints[i][j]->setPosition(newPosition);
			}
			if(j == 0)
			{
				gridPoints[i][j]->setLocked(true);
				newPosition.setValues(newPosition.x, bottom);
				gridPoints[i][j]->setPosition(newPosition);
			}
			else if(j == numberOfGridPoints - 1)
			{
				gridPoints[i][j]->setLocked(true);
				newPosition.setValues(newPosition.x, top);
				gridPoints[i][j]->setPosition(newPosition);
			}

		}
	}	
};

FieldC::~FieldC()
{
};

void FieldC::update(DWORD milliseconds)
{
	switch(StateManagerC::GetInstance()->getState())
	{
		case StateManagerC::MAIN_MENU:
		case StateManagerC::SCORE_MENU:
				gridUpdate(milliseconds);
				break;
		case StateManagerC::GAMEPLAY:
				gridUpdate(milliseconds);
				break;
	}
};

void FieldC::gridUpdate(DWORD milliseconds)
{
	Vector3C velocityChange;
	Vector3C colorChange;
	//Update Grid
	for(int32_t i = 0; i < numberOfGridPoints; i++)
	{
		for(int32_t j = 0; j < numberOfGridPoints; j++)
		{
			//Decrease the color of the grid
			gridPoints[i][j]->setNumberOfColorSums(1.0f);

			//A
			Vector3C currentPositionA = gridPoints[i][j]->getCurrentPosition();
			Vector3C initalPositionA = gridPoints[i][j]->getInitalPosition();
			//B
			Vector3C currentPositionB;
			Vector3C initalPositionB;

			float normalDistance = 0;
			float currentDistance = 0;
			
			colorChange.setValues(0.0f, 0.0f, 0.0f);
			colorChange.setEqualToVector(*gridPoints[i][j]->getColorChange());
			velocityChange.setValues(0,0);
			
			//Horizontal Lines
			if(i - 1 >= 0){
				currentPositionB = gridPoints[i - 1][j]->getCurrentPosition();
				initalPositionB = gridPoints[i - 1][j]->getInitalPosition();

				normalDistance = initalPositionA.x - initalPositionB.x;
				currentDistance = currentPositionA.x - currentPositionB.x;

				velocityChange.addXComponet(-(currentDistance - normalDistance)/400);
				//Merge the colors
				gridPoints[i][j]->setNumberOfColorSums(gridPoints[i][j]->returnNumberOfColorSums() + 1.0f);
				colorChange.addVector(*gridPoints[i - 1][j]->getCurrentColor());
			}
			if(i + 1 < numberOfGridPoints){
				currentPositionB = gridPoints[i + 1][j]->getCurrentPosition();
				initalPositionB = gridPoints[i + 1][j]->getInitalPosition();

				normalDistance = initalPositionA.x - initalPositionB.x;
				currentDistance = currentPositionA.x - currentPositionB.x;

				velocityChange.addXComponet(-(currentDistance - normalDistance)/400);
				//Merge the colors
				gridPoints[i][j]->setNumberOfColorSums(gridPoints[i][j]->returnNumberOfColorSums() + 1.0f);
				colorChange.addVector(*gridPoints[i + 1][j]->getCurrentColor());
			}

			//Vertical Lines
			if(j - 1 >= 0){
				currentPositionB = gridPoints[i][j - 1]->getCurrentPosition();
				initalPositionB = gridPoints[i][j - 1]->getInitalPosition();

				normalDistance = initalPositionA.y - initalPositionB.y;
				currentDistance = currentPositionA.y - currentPositionB.y;

				velocityChange.addYComponet(-(currentDistance - normalDistance)/400);
				//Merge the colors
				gridPoints[i][j]->setNumberOfColorSums(gridPoints[i][j]->returnNumberOfColorSums() + 1.0f);
				colorChange.addVector(*gridPoints[i][j - 1]->getCurrentColor());
			}
			if(j + 1 < numberOfGridPoints){
				currentPositionB = gridPoints[i ][j + 1]->getCurrentPosition();
				initalPositionB = gridPoints[i ][j + 1]->getInitalPosition();

				normalDistance = initalPositionA.y - initalPositionB.y;
				currentDistance = currentPositionA.y - currentPositionB.y;

				velocityChange.addYComponet(-(currentDistance - normalDistance)/400);
				//Merge the colors
				gridPoints[i][j]->setNumberOfColorSums(gridPoints[i][j]->returnNumberOfColorSums() + 1.0f);
				colorChange.addVector(*gridPoints[i][j + 1]->getCurrentColor());
			}

			colorChange.scaleVector(1.0f/gridPoints[i][j]->returnNumberOfColorSums());
			gridPoints[i][j]->setColorChange(&colorChange);

			gridPoints[i][j]->applyForce(velocityChange);

			gridPoints[i][j]->update(milliseconds);
		}
	}
	//update grid colors
	for(int32_t i = 0; i < numberOfGridPoints; i++)
	{
		for(int32_t j = 0; j < numberOfGridPoints; j++)
		{
			gridPoints[i][j]->updateColor();
		}
	}
}

void FieldC::render()
{
	
	renderGrid();
	renderEdges();
};


void FieldC::applyPush(Vector3C* position, float_t radius, float_t amount, Vector3C* color)
{
	for(int32_t i = 0; i < numberOfGridPoints; i++)
	{
		for(int32_t j = 0; j < numberOfGridPoints; j++)
		{
			Vector3C gridPointPosition = gridPoints[i][j]->getCurrentPosition();

			if(gridPointPosition.distance(*position) < radius)
			{
				//Push the grid point
				Vector3C pushDirection;
				pushDirection.x = position->x - gridPointPosition.x;
				pushDirection.y = position->y - gridPointPosition.y;
				pushDirection.scaleVector(amount);
				gridPoints[i][j]->applyForce(pushDirection);

				//Color the grid
				gridPoints[i][j]->appyPushColorChange(color);
			}
		}
	}
}

void FieldC::applyGravityPull(Vector3C* position, float_t radius, float_t amount, Vector3C* color)
{
	for(int32_t i = 0; i < numberOfGridPoints; i++)
	{
		for(int32_t j = 0; j < numberOfGridPoints; j++)
		{
			Vector3C gridPointPosition = gridPoints[i][j]->getCurrentPosition();

			if(gridPointPosition.distance(*position) < radius){
				//float_t strength

				//Push the grid point
				Vector3C pullDirection;
				pullDirection.x = position->x - gridPointPosition.x;
				pullDirection.y = position->y - gridPointPosition.y;
				pullDirection.scaleVector(amount);

				//Clear the velocity of the grid the point so it seems like we are holding the point
				gridPoints[i][j]->clearVelocity();
				gridPoints[i][j]->applyForce(pullDirection);

				//Color the grid
				gridPoints[i][j]->appyPushColorChange(color);

			}
		}
	}
}

void FieldC::renderGrid()
{
	glEnable (GL_BLEND);
	glBlendFunc (GL_SRC_ALPHA, GL_ZERO);


	glLineWidth(1.0f); 

	//Render Grid
	for(int32_t i = 0; i < numberOfGridPoints; i++)
	{
		for(int32_t j = 0; j < numberOfGridPoints; j++)
		{
			Vector3C temporaryPosition = gridPoints[i][j]->getCurrentPosition();

			if(CameraManagerC::GetInstance()->checkIfInViewport(temporaryPosition)){ //Only draw if it's in the current camera view

				Vector3C temporaryVelocity = gridPoints[i][j]->getCurrentVelocity();
#if(DRAW_ONLY_EFFECTED)
				if(temporaryVelocity.x >= DRAW_EFFECTED_THRESHOLD || temporaryVelocity.x <= -DRAW_EFFECTED_THRESHOLD || temporaryVelocity.y >= DRAW_EFFECTED_THRESHOLD 
					|| temporaryVelocity.y <=  -DRAW_EFFECTED_THRESHOLD)
				{
					//Point
					DrawCircle(5, temporaryPosition.x, temporaryPosition.y, gridPoints[i][j]->getCurrentColor(), true);

					//Lines
					Vector3C lineBPosition;
					//Horizontal Lines
					if(i - 1 >= 0){
						lineBPosition = gridPoints[i - 1][j]->getCurrentPosition();
						DrawLine(temporaryPosition.x, temporaryPosition.y, lineBPosition.x, lineBPosition.y, gridPoints[i][j]->getCurrentColor());
					}
					if(i + 1 < numberOfGridPoints){
						lineBPosition = gridPoints[i + 1][j]->getCurrentPosition();
						DrawLine(temporaryPosition.x, temporaryPosition.y, lineBPosition.x, lineBPosition.y, gridPoints[i][j]->getCurrentColor());
					}

					//Vertical Lines
					if(j - 1 >= 0){
						lineBPosition = gridPoints[i][j - 1]->getCurrentPosition();
						DrawLine(temporaryPosition.x, temporaryPosition.y, lineBPosition.x, lineBPosition.y, gridPoints[i][j]->getCurrentColor());
					}
					if(j + 1 < numberOfGridPoints){
						lineBPosition = gridPoints[i][j + 1]->getCurrentPosition();
						DrawLine(temporaryPosition.x, temporaryPosition.y, lineBPosition.x, lineBPosition.y, gridPoints[i][j]->getCurrentColor());
					}
				}
#else
				//Point
				DrawCircle(5, temporaryPosition.x, temporaryPosition.y, r, g, b, true);

				//Lines
				Vector3C lineBPosition;
				//Horizontal Lines
				if(i - 1 > 0){
					lineBPosition = gridPoints[i - 1][j]->getCurrentPosition();
					DrawLine(temporaryPosition.x, temporaryPosition.y, lineBPosition.x, lineBPosition.y, r, g, b);
				}
				if(i + 1 < numberOfGridPoints){
					lineBPosition = gridPoints[i + 1][j]->getCurrentPosition();
					DrawLine(temporaryPosition.x, temporaryPosition.y, lineBPosition.x, lineBPosition.y, r, g, b);
				}

				//Vertical Lines
				if(j - 1 > 0){
					lineBPosition = gridPoints[i][j - 1]->getCurrentPosition();
					DrawLine(temporaryPosition.x, temporaryPosition.y, lineBPosition.x, lineBPosition.y, r, g, b);
				}
				if(j + 1 < numberOfGridPoints){
					lineBPosition = gridPoints[i ][j + 1]->getCurrentPosition();
					DrawLine(temporaryPosition.x, temporaryPosition.y, lineBPosition.x, lineBPosition.y, r, g, b);
				}
#endif
			}
		}
	}
}

void FieldC::renderEdges()
{
	//Render the edges of the grid
	glLineWidth(1.5f); 

	char8_t r = (char8_t)(mFieldColor>>16 & 0xFF);
	char8_t g = (char8_t)(mFieldColor>>8 & 0xFF);
	char8_t b = (char8_t)(mFieldColor>>0 & 0xFF);

	//Inside Edge
	float_t left = mPosition.x - (float_t)mWidth/2.0f;
	float_t right = mPosition.x + (float_t)mWidth/2.0f;
	float_t bottom = mPosition.y - (float_t)mHeight/2.0f;
	float_t top = mPosition.y + (float_t)mHeight/2.0f;

	DrawLine(left,top,right,top,r,g,b);
	DrawLine(right,top,right,bottom,r,g,b);
	DrawLine(right,bottom,left,bottom,r,g,b);
	DrawLine(left,bottom,left,top,r,g,b);

	//Outside Edge
	glLineWidth(1.0f); 
	float_t outEdgeSpacing = 20.0f;

	left = mPosition.x - (float_t)mWidth/2.0f - outEdgeSpacing;
	right = mPosition.x + (float_t)mWidth/2.0f + outEdgeSpacing;
	bottom = mPosition.y - (float_t)mHeight/2.0f - outEdgeSpacing;
	top = mPosition.y + (float_t)mHeight/2.0f + outEdgeSpacing;

	DrawLine(left,top,right,top,r,g,b);
	DrawLine(right,top,right,bottom,r,g,b);
	DrawLine(right,bottom,left,bottom,r,g,b);
	DrawLine(left,bottom,left,top,r,g,b);
}

void FieldC::resetGrid()
{
	//Render Grid
	for(int32_t i = 0; i < numberOfGridPoints; i++)
	{
		for(int32_t j = 0; j < numberOfGridPoints; j++)
		{
			gridPoints[i][j]->setPosition(gridPoints[i][j]->getInitalPosition());
			gridPoints[i][j]->clearVelocity();
			gridPoints[i][j]->setColor(new Vector3C(0.0f, 0.0f, 0.0f));
		}
	}
}

void FieldC::applyColor(Vector3C* position, float_t radius, Vector3C* color)
{
	for(int32_t i = 0; i < numberOfGridPoints; i++)
	{
		for(int32_t j = 0; j < numberOfGridPoints; j++)
		{
			Vector3C gridPointPosition = gridPoints[i][j]->getCurrentPosition();

			if(gridPointPosition.distance(*position) < radius)
			{
				//Color the grid
				gridPoints[i][j]->appyPushColorChange(color);
			}
		}
	}
}