#define GAME_CPP
#include <windows.h>											
#include <stdio.h>												
#include <stdarg.h>											
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include "glew.h"
#include "gl\gl.h"												
#include "gl\glu.h"												
#include "gl\glut.h"
#include "wglew.h"
#include "glext.h"
#include "wglext.h"
#include "gl3.h"
#include "baseTypes.h"
#include "openglframework.h"
#include "collInfo.h"
#include "object.h"
#include "enemy.h"
#include "player.h"
#include "field.h"
#include "random.h"
#include "openGLStuff.h"
#include "game.h"
#include "StateManager.h"
#include "PlayerManager.h"
#include "EnemyManager.h"
#include "FieldManager.h"
#include "InputManager.h"
#include "SpriteManager.h"
#include "ParticleManager.h"
#include "SoundManager.h"
#include "GUIManager.h"
#include "PickUpManager.h"
#include "ShapeDraw.h"
#include "CameraManager.h"
#include "MenuManager.h"

// Declarations
const char8_t CGame::mGameTitle[]="Geometry Wars";
CGame* CGame::sInstance=NULL;
BOOL Initialize (GL_Window* window, Keys* keys)					// Any OpenGL Initialization Goes Here
{
	initOpenGLDrawing(window,keys,0.0f, 0.0f, 0.0f);
	CGame::CreateInstance();
	CGame::GetInstance()->init();

	return TRUE;						
}

void CGame::init()
{
	_CrtSetDbgFlag ( _CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF );

	//Creates
	GUIManagerC::CreateInstance();
	EnemyManagerC::CreateInstance();
	PlayerManagerC::CreateInstance();
	ParticleManagerC::CreateInstance();
	SoundManagerC::CreateInstance();
	StateManagerC::CreateInstance();
	FieldManagerC::CreateInstance();
	InputManagerC::CreateInstance();
	SpriteManagerC::CreateInstance();
	PickUpManagerC::CreateInstance();
	CameraManagerC::CreateInstance();
	MenuManagerC::CreateInstance();

	//Inits
	GUIManagerC::GetInstance()->init();
	SoundManagerC::GetInstance()->init();
	InputManagerC::GetInstance()->init();
	InputManagerC::GetInstance()->setScreenInformation(mScreenWidth, mScreenHeight, 2000, 2000);

	EnemyManagerC::GetInstance()->init();
	PlayerManagerC::GetInstance()->init();
	ParticleManagerC::GetInstance()->init();
	StateManagerC::GetInstance()->setState(StateManagerC::MAIN_MENU);
	FieldManagerC::GetInstance()->init();
	SpriteManagerC::GetInstance()->init(28,26);
	PickUpManagerC::GetInstance()->init();
	CameraManagerC::GetInstance()->init();
	MenuManagerC::GetInstance()->init();



	/**HANDLE hLogFile;
	hLogFile = CreateFileA("c:\\memoryLeaksDump.rtf", GENERIC_WRITE,FILE_SHARE_WRITE, NULL, CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL, NULL);

	 _CrtSetReportMode(_CRT_WARN, _CRTDBG_MODE_FILE);
	 _CrtSetReportMode(_CRT_ERROR, _CRTDBG_MODE_FILE);
	 _CrtSetReportMode(_CRT_ASSERT, _CRTDBG_MODE_FILE);
	 _CrtSetReportFile(_CRT_WARN, hLogFile);
	 _CrtSetReportFile(_CRT_ERROR, hLogFile);
	 _CrtSetReportFile(_CRT_ASSERT, hLogFile);

	 _CrtSetDbgFlag( _CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF);*/

}
void CGame::UpdateFrame(DWORD milliseconds)			
{

	switch(StateManagerC::GetInstance()->getState())
	{
		case StateManagerC::MAIN_MENU:
		case StateManagerC::SCORE_MENU:
			CameraManagerC::GetInstance()->setCameraPosition(Vector3C(0, 0, 0));
			MenuManagerC::GetInstance()->update(milliseconds);
			ParticleManagerC::GetInstance()->updateParticles(milliseconds);
			FieldManagerC::GetInstance()->updateField(milliseconds);
			SoundManagerC::GetInstance()->update();
			break;
		case StateManagerC::GAMEPLAY:
		case StateManagerC::GAME_PAUSED:
			//Update Camera
			updateCamera();

			//Handle Input
			InputManagerC::GetInstance()->update();
			keyProcess();
			//Pause the game if needed
			if(InputManagerC::GetInstance()->previousKeys['P']){
				if(InputManagerC::GetInstance()->currentKeys['P'] == false){
					StateManagerC::GetInstance()->togglePause();
				}
			}

			//Sounds
			SoundManagerC::GetInstance()->update();

			EnemyManagerC::GetInstance()->updateEnemies(milliseconds);
			PlayerManagerC::GetInstance()->updatePlayers(milliseconds);
			ParticleManagerC::GetInstance()->updateParticles(milliseconds);
			
			PickUpManagerC::GetInstance()->updatePickUps(milliseconds);

			GUIManagerC::GetInstance()->updateGUI(milliseconds);
			FieldManagerC::GetInstance()->updateField(milliseconds);
			break;
	}
	
}

void CGame::DrawScene(void)											
{
	startOpenGLDrawing();
	glLoadIdentity();
	
	wasLastFrameSkipped = false;
		

	//Clear the screen
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	switch(StateManagerC::GetInstance()->getState())
	{
		//Push Matrix
		glPushMatrix();

		case StateManagerC::SCORE_MENU:
		case StateManagerC::MAIN_MENU:
			glPushMatrix();
			FieldManagerC::GetInstance()->renderField();
			ParticleManagerC::GetInstance()->renderParticles();
			MenuManagerC::GetInstance()->render();
			//Pop Matrix
			glPopMatrix();
			break;
		case StateManagerC::GAMEPLAY:
		case StateManagerC::GAME_PAUSED:
			//Push Matrix
			glPushMatrix();

			//Move the camera to the correct scrolling position
			glTranslatef(-CameraManagerC::GetInstance()->getCameraPosition().x, -CameraManagerC::GetInstance()->getCameraPosition().y, CameraManagerC::GetInstance()->getCameraPosition().z);
		
			FieldManagerC::GetInstance()->renderField();
			InputManagerC::GetInstance()->draw();
			ParticleManagerC::GetInstance()->renderParticles();
			PickUpManagerC::GetInstance()->renderPickUps();
			EnemyManagerC::GetInstance()->renderEnemys();
			PlayerManagerC::GetInstance()->renderPlayers();	
			GUIManagerC::GetInstance()->renderGUI();

			//Pop Matrix
			glPopMatrix();
			break;

		//Pop Matrix
		glPopMatrix();
	}

	endOpenGLDrawing();
}

CGame *CGame::CreateInstance()
{
	sInstance = new CGame();
	return sInstance;
}

void CGame::shutdown()
{
	SoundManagerC::GetInstance()->shutdown();
	EnemyManagerC::GetInstance()->shutdown();
	StateManagerC::GetInstance()->shutdown();
	FieldManagerC::GetInstance()->shutdown();
	SpriteManagerC::GetInstance()->shutdown();
	ParticleManagerC::GetInstance()->shutdown();
	PickUpManagerC::GetInstance()->shutdown();
	GUIManagerC::GetInstance()->shutdown();

}

void CGame::DestroyGame(void)
{
	//delete EnemyManagerC::GetInstance();	
	delete StateManagerC::GetInstance();	
	delete FieldManagerC::GetInstance();
	delete SoundManagerC::GetInstance();
	delete SpriteManagerC::GetInstance();
	delete ParticleManagerC::GetInstance();
	delete GUIManagerC::GetInstance();
}

char *CGame::textFileRead(char *fn) 
{
	FILE *fp;
	char *content = NULL;

	int count=0;

	if (fn != NULL) {
		fopen_s(&fp,fn,"rt");

		if (fp != NULL) {
      
      fseek(fp, 0, SEEK_END);
      count = ftell(fp);
      rewind(fp);

			if (count > 0) {
				content = (char *)malloc(sizeof(char) * (count+1));
				count = fread(content,sizeof(char),count,fp);
				content[count] = '\0';
			}
			fclose(fp);
		}
	}
	return content;
}

int CGame::textFileWrite(char *fn, char *s) 
{

	FILE *fp;
	int status = 0;

	if (fn != NULL) {
		fopen_s(&fp,fn,"w");

		if (fp != NULL) {
			
			if (fwrite(s,sizeof(char),strlen(s),fp) == strlen(s))
				status = 1;
			fclose(fp);
		}
	}
	return(status);
}

void CGame::updateCamera()
{
	Vector3C myPlayerPosition = *PlayerManagerC::GetInstance()->getPlayerPtr(0)->getPosition();

	//Getting the field width and height
	float_t fieldWidth = ((float_t)FieldManagerC::GetInstance()->getFieldPtr()->getWidth())/2.0f;
	float_t fieldHeight = ((float_t)FieldManagerC::GetInstance()->getFieldPtr()->getHeight())/2.0f;

	float_t xCameraPosition = 0;
	float_t yCameraPosition = 0;

	//Horizontal Scrolling
	if(fieldWidth - (fabs((float_t)myPlayerPosition.x)) < fieldWidth/1.5f)
		xCameraPosition = -(myPlayerPosition.x/fabs(myPlayerPosition.x)) *(fieldWidth - fieldWidth/1.5f);
	else
		xCameraPosition = -myPlayerPosition.x;
	//Vertical Scrolling
	if(fieldHeight - (fabs((float_t)myPlayerPosition.y)) < fieldHeight/1.5f)
			yCameraPosition = -(myPlayerPosition.y/fabs(myPlayerPosition.y)) *(fieldHeight - fieldHeight/1.5f);
	else
		yCameraPosition = -myPlayerPosition.y;
	CameraManagerC::GetInstance()->setCameraPosition(Vector3C(-xCameraPosition, -yCameraPosition, 100.0f));//CameraManagerC::GetInstance()->getCameraPosition().z));
}