/*************************************************
**Authors: Kevin Cartrette, Alex Lemke			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary: Player class which handles players	**
**movement, bombs and bullets.					**
*************************************************/

#include <windows.h>											
#include <stdio.h>											
#include <stdarg.h>												
#include <math.h>												
#include "gl\gl.h"												
#include "gl\glu.h"												
#include "gl\glut.h"
#include "baseTypes.h"
#include "ShapeDraw.h"
#include "collInfo.h"
#include "object.h"
#include "player.h"
#include "field.h"
#include "fieldmanager.h"
#include "random.h"
#include "stateManager.h"
#include "inputmanager.h"
#include "EnemyManager.h"
#include "SpriteManager.h"
#include "Bomb.h"
#include "gamePlayVariables.h"

PlayerC::PlayerC(float initPosX,float initPosY,float initVelX,float initVelY,int id)
{
	mEnabled = false;
	invincible = true;

	mPosition.x = initPosX;
	mPosition.y = initPosY;
	mVelocity.x = initVelX;
	mVelocity.y = initVelY;
	//Half size player sprite.
	mRadius = SHIP_RADIUS;
	lives = NUMBER_START_LIVES;
	//Sprite
	mySprite.setUp(1, 0,0,0, 64 * 2, 64 * 2,0,0,0,64,64);
	mySprite.setCentered(true);

	//Bullets
	bulletPtrs = (BulletC**)malloc(NUM_BULLETS*sizeof(BulletC*));
	for(int32_t b = 0; b < NUM_BULLETS; b++)
    {
		bulletPtrs[b] = new BulletC();
    }

	numberBombs = NUMBER_BOMBS;

	currentTime = 0;
	timeBetweenUpdates = 0;

	invincibleTime = INVINCIBLE_DELAYTIME;
	timeSinceInvincible = 0;

	bombDelay = BOMB_DELAYTIME;
	timeSinceLastBomb = 0;

	currentShotTimer = 0;
	shotTimerLimit = 3;

	//Score
	score = 0;
	multiplier = 1.0f;
	
	mVelocity.x = 0;
	mVelocity.y = 0;

	playerColor.x = 0.3f;
	playerColor.y = 0.3f;
	playerColor.z = 0.3f;
}

void PlayerC::reset()
{
	for(int i = 0; i < NUM_BULLETS; i ++)
	{
		bulletPtrs[i]->disable();
	}

	invincible = true;

	mPosition.x = 0;
	mPosition.y = 0;
	mVelocity.x = 0;
	mVelocity.y = 0;

	lives = NUMBER_START_LIVES;
	//Sprite
	mySprite.setCentered(true);

	numberBombs = NUMBER_BOMBS;

	currentTime = 0;
	timeBetweenUpdates = 0;

	invincibleTime = INVINCIBLE_DELAYTIME;
	timeSinceInvincible = 0;

	bombDelay = BOMB_DELAYTIME;
	timeSinceLastBomb = 0;

	currentShotTimer = 0;
	shotTimerLimit = 3;
	score = 0;
	multiplier = 1.0f;
	
	mVelocity.x = 0;
	mVelocity.y = 0;
}

PlayerC::PlayerC()
{
    //mEnabled = true;
    mCollInfo.shape = CollInfoC::SHAPE_CIRCLE;

	bulletPtrs = (BulletC**)malloc(NUM_BULLETS*sizeof(BulletC*));
	for(int32_t b = 0; b < NUM_BULLETS; b++)
    {
		bulletPtrs[b] = new BulletC();
    }
	mVelocity.x = 0;
	mVelocity.y = 0;
};
PlayerC::~PlayerC()
{
};

void PlayerC::update(DWORD milliseconds)
{
	if(mEnabled){
		switch(StateManagerC::GetInstance()->getState())
		{
			case StateManagerC::GAMEPLAY:

				currentTime += (double)milliseconds;
				if(currentTime >= timeBetweenUpdates)
				{

					//Timer logic to slow down update of player needs to go here as well, update every other frame probably better.
					timeSinceLastBomb += (double)milliseconds;
					timeSinceInvincible += (double)milliseconds;

					if(timeSinceInvincible >= invincibleTime)
					{
						invincible = false;
					}

					Vector3C playerPosition;
					playerPosition.x = mPosition.x;
					playerPosition.y = mPosition.y;
					FieldManagerC::GetInstance()->applyPush(&playerPosition, 50, 0.005f, (&playerColor));

					//Movement
					Vector3C velocityChange;
					velocityChange.x = 0;
					velocityChange.y = 0;
					//Horizontal
					if(InputManagerC::GetInstance()->currentKeys['A']){
						velocityChange.x -= 1.0f;
					}
					else if(InputManagerC::GetInstance()->currentKeys['D']){
						velocityChange.x += 1.0f;
					}
					//Vertical
					if(InputManagerC::GetInstance()->currentKeys['W']){
						velocityChange.y += 1.0f;
					}
					else if(InputManagerC::GetInstance()->currentKeys['S']){
						velocityChange.y -= 1.0f;
					}
					if(velocityChange.getLength() > 0){
						velocityChange.normalize();
						velocityChange.scaleVector(0.35f);
						mVelocity.add2DQuickVector(velocityChange);
					}

					//Bomb
					if(InputManagerC::GetInstance()->currentKeys[' ']) //Space to handle firing of bomb.
					{
						if(timeSinceLastBomb >= bombDelay && getNumberBombs() > 0)
						{
							bombsVector.push_back(new BombC);
							FieldManagerC::GetInstance()->applyPush(&mPosition, 300, 0.15f, (&playerColor));
							int bombSpot = bombsVector.size() - 1;
							bombsVector[bombSpot]->init(*getPosition(),*getVelocity(),1000,200);
							numberBombs--;
							timeSinceLastBomb = 0;
						}
					}
					//Update the bomb vectors.
					bool isAlive;
					for (bombsIterator = bombsVector.begin(); bombsIterator != bombsVector.end();)
					{
						isAlive = (*bombsIterator)->update((double)milliseconds);
						if(isAlive != true)
						{
							(*bombsIterator)->shutdown();
							bombsIterator = bombsVector.erase(bombsIterator);
						}
						else
						{
							(*bombsIterator)->update((double)milliseconds);
							bombsIterator++;
						}
					}

					//Limit the players max velocity.
					if(mVelocity.getLength() > MAX_VELOCITY)
					{
						mVelocity.normalize();
						mVelocity.scaleVector(MAX_VELOCITY);
					}
					//If holding left mouse button down then player is firing bullets.
					if(InputManagerC::GetInstance()->isLeftMouseButtonDown()){
						currentShotTimer++;
						if(currentShotTimer > shotTimerLimit)
						{
							currentShotTimer = 0;
							//Fire a bullet!
							Vector2C mousePosition = InputManagerC::GetInstance()->returnMouseGamePosition();
	
							Vector3C bulletDirection;
							bulletDirection.x = (mousePosition.x - mPosition.x);
							bulletDirection.y = (mousePosition.y - mPosition.y);

							bulletDirection.normalize();
							Vector3C newBulletDirection;

							int bulletNumber = 0;
							for(int b = 0; b < NUM_BULLETS; b++)
							{
								if(bulletPtrs[b]->getEnabled() == false)
								{
									bulletPtrs[b]->enable();
									bulletPtrs[b]->setPosition(mPosition.x, mPosition.y);

									//Scale Velocity
									bulletDirection.scaleVector(BULLET_SCALE_FACTOR);
									bulletPtrs[b]->setVelocity(bulletDirection);

									newBulletDirection.x = bulletDirection.x;
									newBulletDirection.y = bulletDirection.y;

									if(bulletNumber == 1)
									{
										newBulletDirection.rotateAroundZAxis(BULLET_RATE_ROTATION);
										newBulletDirection.normalize();
										newBulletDirection.scaleVector(2.0f);
										bulletPtrs[b]->setVelocity(newBulletDirection);
									}
									else if(bulletNumber == 2)
									{
										newBulletDirection.rotateAroundZAxis(-BULLET_RATE_ROTATION);
										newBulletDirection.normalize();
										newBulletDirection.scaleVector(BULLET_SCALE_FACTOR);
										bulletPtrs[b]->setVelocity(newBulletDirection);
									}

									bulletNumber++;
									if(bulletNumber == 3)
										b = NUM_BULLETS; //Early Out of loop
								}
							}
						}
					}

					//Update all bullets posiitons //One check to see if any bullets are active, rather then all these compares.
					for(int b = 0; b < NUM_BULLETS; b++){
						bulletPtrs[b]->update(milliseconds);
					}

					//Apply friction to the player.
					mVelocity.x *= PLAYER_FRICTION;
					mVelocity.y *= PLAYER_FRICTION;
					if(mVelocity.getLength() < 0.01)
						mVelocity.scaleVector(0.0f);

					//Increase Position.
					mPosition.x += mVelocity.x*milliseconds;
					mPosition.y += mVelocity.y*milliseconds;

					//Set Sprite Position.
					mySprite.setPosition(mPosition.x, mPosition.y, 0.0f);

					//Setting Rotation.
					if(mVelocity.getLength() != 0)
					{
						Vector3C straightRight;
						straightRight.setValues(1.0f, 0.0f);
						Vector3C velocityNormalzied;
						velocityNormalzied.setValues(mVelocity.x, mVelocity.y);
				
						velocityNormalzied.normalize();
				
						float_t angle = velocityNormalzied.angleBetween(straightRight);

						float negate = 1.0f;

						if(mVelocity.y < 0.0f)
							mySprite.setRotation(angle * (-HALF_DEGREES_OF_CIRCLE) / PI_DEFINE);
						else	
							mySprite.setRotation(angle * HALF_DEGREES_OF_CIRCLE / PI_DEFINE);
					}

					currentTime = 0;
				}
				break;
		}
	}
}

bool PlayerC::DoCollisions()
{
	bool hitHappen = false;
	CollideField();
#if (GOD_MODE == 1)
	invincible = true;
#endif
	if(invincible == true)
	{
		EnemyManagerC::GetInstance()->checkEnemyRadialCollision(mRadius, this->getID(), true);
		//EnemyManagerC::GetInstance()->checkEnemyRadialCollision(mRadius, mPosition, false);
	}
	else
	{
		hitHappen = EnemyManagerC::GetInstance()->checkEnemyRadialCollision(mRadius,this->getID() ,false);//EnemyManagerC::GetInstance()->checkEnemyRadialCollision(mRadius, mPosition, false);
	}
	
	return hitHappen;
}
void PlayerC::CollideField(void)
{
	FieldC *field = FieldManagerC::GetInstance()->getFieldPtr();

	float_t rightSide = field->getPosition()->x+((float_t)field->getWidth()/TWO);
	float_t leftSide = field->getPosition()->x-((float_t)field->getWidth()/TWO);
	float_t topSide = field->getPosition()->y+((float_t)field->getHeight()/TWO);
	float_t bottomSide = field->getPosition()->y-((float_t)field->getHeight()/TWO);
	
	//Box collision
	if(mPosition.x- mRadius <= leftSide  )
	{
		mVelocity.x = -mVelocity.x;
		mPosition.x = leftSide + mRadius;
		setRandomColor();
	}
	if(mPosition.x+ mRadius >= rightSide  )
	{
		mVelocity.x = -mVelocity.x;
		mPosition.x = rightSide - mRadius;
		setRandomColor();
	}
	if(mPosition.y- mRadius <= bottomSide  )
	{
		mVelocity.y = -mVelocity.y;
		mPosition.y = bottomSide + mRadius;
		setRandomColor();
	}
	if(mPosition.y+ mRadius >= topSide  )
	{
		mVelocity.y = -mVelocity.y;
		mPosition.y = topSide - mRadius;
		setRandomColor();
	}
};

int32_t PlayerC::getNumberBombs()
{
	return numberBombs;
}

int32_t PlayerC::getLives()
{
	return lives;
}
int32_t PlayerC::subtractLife()
{
	return lives--;
}
int32_t PlayerC::addLife()
{
	return lives++;
}

void PlayerC::respawn()
{
	mPosition.x = 0;
	mPosition.y = 0;
	mVelocity.x = 0;
	mVelocity.y = 0;
	//mySprite.setUp(1, 0,0,0, 64 * 2, 64 * 2,0,0,0,64,64);
	mySprite.setCentered(true);
}

void PlayerC::render()
{
	if(mEnabled){
		//Push Matrix
		glPushMatrix();

		SpriteManagerC::GetInstance()->drawSprite(mySprite);
		//Render Sheild
		if(invincible == true)
		{
			DrawCircle(mRadius*MODIFER_FOR_SHEILD_RADIUS,mPosition.x, mPosition.y,1.0f,1.0f,1.0f,false);
		}

		//Pop Matrix
		glPopMatrix();
	}

	//Bullets
	for(int32_t b = 0; b < NUM_BULLETS; b++)
	{
		bulletPtrs[b]->render();
	}

	//Bombs!
	for (bombsIterator = bombsVector.begin(); bombsIterator != bombsVector.end(); bombsIterator++)
	{
		(*bombsIterator)->render();
	}
}

void PlayerC::setRandomColor()
{
	mPlayerColor = getRangedRandom(0,256);
	mPlayerColor += getRangedRandom(0,256) << 8;
	mPlayerColor += getRangedRandom(0,256) << 16;
}


BulletC *PlayerC::getBulletPtr(int32_t index)
{
	return bulletPtrs[index];
}

void PlayerC::death()
{
	numberBombs = NUMBER_BOMBS;
}

//Score
void PlayerC::increaseScore(int32_t pointValue)
{
	score += (int32_t)(pointValue * multiplier);
}

int32_t PlayerC::getScore()
{
	return(score);
}
void PlayerC::increaseMultiplier(int32_t increaseAmount)
{
	multiplier += increaseAmount;
}

int32_t PlayerC::getMultiplier()
{
	return(multiplier);
}

void  PlayerC::setInvincible(bool setInvincible)
{
	invincible = setInvincible;
	//If player was set to invincible restart counter.
	if(setInvincible == true)
	{
		timeSinceInvincible = 0;
	}
}

bool  PlayerC::isInvincible()
{
	return invincible;
}

void PlayerC::shutdown()
{
	for (bombsIterator = bombsVector.begin(); bombsIterator != bombsVector.end();)
	{
		delete(*bombsIterator);
		bombsIterator = bombsVector.erase(bombsIterator);
	}
}

void PlayerC::applyForce(Vector3C pushDirection)
{
	//mVelocity.scaleVector(1.0f - (1.0f/pushDirection.getLength()));
	mVelocity.add2DQuickVector(pushDirection);
}

Vector3C* PlayerC::returnColor()
{
	return(&playerColor);
}