/*************************************************
**Authors: Kevin Cartrette, Alex Lemke			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary: Player class which handles players	**
**movement, bombs and bullets.					**
*************************************************/

#ifndef PLAYER_H
#define PLAYER_H

#include "Sprite.h"
#include "ParticleManager.h"
#include "Object.h"
#include "Bullet.h"
#include "Bomb.h"
#include <vector>

using namespace std;

/*************************************************
**Summary: Player class which controls players  **
**actions.										**
**Authors: Kevin Cartrette, Alex Lemke			**
*************************************************/
class PlayerC : public ObjectC
{
public:
    PlayerC();
	PlayerC(float_t initPosX, float_t initPosY, float_t initVelX, float_t initVelY,int id);
    ~PlayerC();
	//Function prototypes.
	void setPlayerColor(long color) {mPlayerColor = color;};
    long getPlayerColor() {return mPlayerColor;};
    void update(unsigned long milliseconds);
    virtual void render();
	bool DoCollisions();
	void reset();
	void death();
	void respawn();
	void shutdown();
    void setRadius(float radius) {mCollInfo.param1 = radius;mRadius = radius;};
	void setRandomColor();
	void reduceVelocity(float_t factor);
	void increaseMultiplier(int32_t increaseAmount);
	void increaseScore(int32_t pointValue);
	void setInvincible(bool setInvincible);
	//Getters.
	int32_t getScore();
	int32_t getLives();
	int32_t subtractLife();
	int32_t addLife();
	int32_t getMultiplier();
	int32_t getNumberBombs();
	//Bombs
	bool isInvincible();
	//Bullets.
	int32_t	getNumBullets() {return NUM_BULLETS;};
	BulletC* getBulletPtr(int32_t index);
	//Pulling/Pushing Player
	void applyForce(Vector3C pushDirection);

	Vector3C* returnColor();

private:
	void CollideField();
	long mPlayerColor;
    float_t mRadius;
	bool mAtRest;
	bool invincible;
	//Player sprite.
	SpriteC mySprite;
	//Bullet control variables.
	int32_t currentShotTimer;
	int32_t shotTimerLimit;
	BulletC **bulletPtrs;
	int32_t numberBombs;
	int32_t lives;
	static const int32_t NUM_BULLETS = 80;
	//Vector of bomb objects.
	vector<BombC*> bombsVector;
	vector<BombC*>::iterator bombsIterator;
	//Time control variables.
	double64_t currentTime;
	double64_t timeBetweenUpdates;
	double64_t invincibleTime;
	double64_t timeSinceInvincible;
	double64_t bombDelay;
	double64_t timeSinceLastBomb;
	//Score variables.
	int32_t score;
	int32_t multiplier;

	Vector3C playerColor;
};
#endif